/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_COND_H_
#define _FLUKE_COND_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>

FLUKE_OB_DEFINE(fluke_cond, FLUKE_COND_SIZE);

struct fluke_cond_state
{
	fluke_hash_t	hash;
	oskit_u32_t	flags;		/* attributes of the cond */

	/* Leave at least one pad word for inheritor link.  */
};
typedef struct fluke_cond_state fluke_cond_state_t;

/* flag values - XXX these aren't used currently */
#define COND_MULTITASK		4

struct fluke_mutex;
struct fluke_ref;

fluke_error_t fluke_cond_create(
	fluke_cond_t		*cond);

fluke_error_t fluke_cond_create_hash(
	fluke_cond_t		*cond,
	fluke_hash_t		*hash);

void fluke_cond_destroy(
	fluke_cond_t		*cond);

void fluke_cond_wait(
	fluke_cond_t		*cond,
	struct fluke_mutex	*mutex);

void fluke_cond_signal(
	fluke_cond_t		*cond);

void fluke_cond_broadcast(
	fluke_cond_t		*cond);

void fluke_cond_reference(
	fluke_cond_t		*cond,
	struct fluke_ref	*out_cond_ref);

void fluke_cond_move(
	fluke_cond_t		*old_va,
	fluke_cond_t		*new_va);

void fluke_cond_get_state(
	fluke_cond_t		*cond,
	fluke_cond_state_t	*out_state,
	struct fluke_ref	*out_inheritor_ref);

void fluke_cond_set_state(
	fluke_cond_t		*cond,
	fluke_cond_state_t	*state,
	struct fluke_ref	*inheritor_ref);


#include <fluke/machine/cond.h>

#endif /* _FLUKE_COND_H_ */
