/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This function parses the argv list handed in and extracts three
 * things:
 *
 * 1) an argv-style list of arguments for the current nester.
 * 2) the name of the child task to spawn.
 * 3) the args to pass to the child (again in, argv-style with argv[0] == name)
 *
 * argv-style means NULL terminated here, as that's what nest_task()
 * wants.
 *
 * [Note: you should pass in the address of a char ** for the out
 * argv's.  The child_task, is the address of a single string.  I
 * over-use "*" so that _you_ can clean-up, if you want.  DOUBLE
 * NOTE:  The arrays are all you have to free, the strings are
 * just copies of the pointers passed in in in_argv[].]
 *
 * Returns TRUE if everything is peachy.  FALSE if black clouds are
 * raining on your parade. (Not really, it'll fail an assert()
 * if somebody messed up.)
 *
 * I assume that the first argument _IS_ the name of the current task.
 *
 * I assume that "our" args are of the form "-foo bar", so
 * the first odd string is the name of the child.
 *
 * e.g. "foo -l 2 -s 0x1024 chkpt obtest" will give 4 args to the        
 * current task ("-l 2 -s 0x1024"), the child will be
 * "chkpt" and invoked with "chkpt obtest" as its argv.
 *
 * Yes, its restrictive.  Deal.  You only have to live
 * with it until Friday...
 *
 * Note: I got the argc and argv in the traditional order. ;-)
 */

#include <malloc.h>
#include <stdio.h>
#include <oskit/boolean.h>
#include <assert.h>

/* Is this somewhere I can get at it? (I mean the real min() function) */
#define pats_min(x,y) (((x) < (y))?(x):(y))

/* Just turn this on to turn on printfs */
static oskit_bool_t debugging = FALSE;

static oskit_bool_t
parse_dem_args(int in_argc, char **in_argv, 
	       int *our_argc, char ***our_argv,
	       char **child_task,
	       int *child_argc, char ***child_argv)
{
	int arg, i;

	/* Assume nothing's gonna happen */
	*our_argv = *child_argv = NULL;
	*child_task = NULL;
	*our_argc = *child_argc = 0;
	arg = 0;

	/*
	 * Make sure there's something to be accomplished here
	 */
	if (debugging) printf("in_argc: %d\n", in_argc);
	if (!in_argc)
		return TRUE;

	/*
	 * Skip our name
	 */
	arg++;

	/*
	 * Get all pairs of "-" style args
	 */
	while (arg < in_argc)
	{
		assert(in_argv[arg]);
		if (debugging) printf("Looking at: %s\n", in_argv[arg]);
		if (in_argv[arg][0] == '-')
		{
			arg += 2;
		}
		else
			break;
	}
	
	/*
	 * _Copy_ our args into our_arg[cv]
	 */
	*our_argc = pats_min(arg, in_argc);
	if (*our_argc)
	{
		*our_argv = malloc(sizeof(char *) * (*our_argc + 1));
		assert(our_argv);
	}
	
	for (i = 0; i < *our_argc; i++)
	{
		if (debugging) printf("Copying in %s\n", in_argv[i]);
		(*our_argv)[i] = in_argv[i];
	}

	if (*our_argc)
		(*our_argv)[*our_argc] = NULL;
	
	/*
	 * Are we done yet?
	 */
	if (arg >= in_argc)
		return TRUE;
	
	/*
	 * Next arg is the child task (which is also child argv[0]).
	 */
	if (debugging) printf("Child task is %s\n", in_argv[arg]);
	assert(in_argv[arg]);
	*child_task = in_argv[arg];

	/*
	 * Remaining args are copied into the
	 * child's args.
	 */	
	*child_argc = in_argc - arg;
	*child_argv = malloc(sizeof(char *) * (*child_argc + 1));
	assert(child_argv);
	
	for (i = 0; arg < in_argc; i++, arg++)
	{
		if (debugging) printf("Copying in arg %s\n", in_argv[arg]);
		assert(in_argv[arg]);
		(*child_argv)[i] = in_argv[arg];
	}

	if (*child_argc)
		(*child_argv)[*child_argc] = NULL;
	
	/*
	 * All done
	 */

	return TRUE;
}
