/*
 * Copyright (c) 1996-1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 *	Machine-independent Fluke exception definitions.
 */
#ifndef	_FLUKE_EXCEPTION_H_
#define	_FLUKE_EXCEPTION_H_


/*** Generic exception classes ***/

#define FLUKE_EXC_BAD_ACCESS		1	/* Could not access memory */
#define FLUKE_EXC_BAD_INSTRUCTION	2	/* Instruction failed */
#define FLUKE_EXC_ARITHMETIC		3	/* Arithmetic exception */
#define FLUKE_EXC_BREAKPOINT		4	/* Trace, breakpoint, etc. */
#define FLUKE_EXC_PAGE_FAULT		5	/* Page fault */
#define FLUKE_EXC_OBJECT_FAULT		6	/* Object fault ("poisoned") */
#define FLUKE_EXC_INSANITY		7	/* Application error detected */


/*** Subcode values that differentiate the various insanity traps ***/

/* General insanity conditions */
#define		FLUKE_INSANITY_NO_OBJECT		0x0101
#define		FLUKE_INSANITY_OBJECT_EXISTS		0x0102
#define		FLUKE_INSANITY_INVALID_OBJECT		0x0103
#define		FLUKE_INSANITY_RACE_CONDITION		0x0104
#define		FLUKE_INSANITY_NOT_CURRENT_THREAD	0x0105
#define		FLUKE_INSANITY_EXCEPTIONS_ENABLED	0x0106
#define		FLUKE_INSANITY_NOT_CURRENT_TASK		0x0107
#define		FLUKE_INSANITY_THREAD_NOT_STOPPED	0x0108
#define		FLUKE_INSANITY_INVALID_ENTRYPOINT	0x0109

#if FLASK
#define		FLUKE_INSANITY_PERMISSION_DENIED	0x01ff
#endif

/* Wrong object type codes; the low 8 bits encodes the expected object type */
#define		FLUKE_INSANITY_WRONG_TYPE		0x0200
#define		FLUKE_INSANITY_NOT_TASK			0x0201
#define		FLUKE_INSANITY_NOT_THREAD		0x0202
#define		FLUKE_INSANITY_NOT_MUTEX		0x0203
#define		FLUKE_INSANITY_NOT_COND			0x0204
#define		FLUKE_INSANITY_NOT_PORT			0x0205
#define		FLUKE_INSANITY_NOT_PSET			0x0206
#define		FLUKE_INSANITY_NOT_CATALOG		0x0207
#define		FLUKE_INSANITY_NOT_REF			0x0208
#define		FLUKE_INSANITY_NOT_MAPPING		0x0209
#define		FLUKE_INSANITY_NOT_REGION		0x020a

/* Wrong reference type codes; the low 8 bits encodes the expected ref type */
/* XXX get rid of these */
#define		FLUKE_INSANITY_WRONG_REF_TYPE		0x0300
#define		FLUKE_INSANITY_NOT_TASK_REF		0x0301
#define		FLUKE_INSANITY_NOT_THREAD_REF		0x0302
#define		FLUKE_INSANITY_NOT_REGION_REF		0x030a
#define		FLUKE_INSANITY_NOT_PORT_REF		0x0305

/* Insanity conditions relating to thread operations */
#define		FLUKE_INSANITY_CURRENT_THREAD		0x0401

/* Insanity conditions relating to VM operations */
#define		FLUKE_INSANITY_INVALID_ADDRESS		0x0501
#define		FLUKE_INSANITY_INVALID_PROT		0x0502
#define		FLUKE_INSANITY_NOT_ALIGNED		0x0503
#define		FLUKE_INSANITY_ADDRESS_OVERLAP		0x0504
#define		FLUKE_INSANITY_INVALID_RANGE		0x0505

/* Insanity conditions relating to IPC operations */
#define		FLUKE_INSANITY_INVALID_DEST		0x0601
#define		FLUKE_INSANITY_MESSAGE_TOO_LARGE	0x0602
#define		FLUKE_INSANITY_BUFFER_TOO_SMALL		0x0603
#define		FLUKE_INSANITY_NOT_SENDER		0x0604
#define		FLUKE_INSANITY_NOT_RECEIVER		0x0605
#define		FLUKE_INSANITY_BUFFER_UNALIGNED		0x0606

/* Insanity conditions relating to synchronization */
#define		FLUKE_INSANITY_MUTEX_LOCKED		0x0701
#define		FLUKE_INSANITY_MUTEX_NOT_LOCKED		0x0702
#define		FLUKE_INSANITY_MUTEX_NOT_OWNER		0x0703
#define		FLUKE_INSANITY_COND_THREADS_WAITING	0x0704
#define		FLUKE_INSANITY_COND_WRONG_MUTEX		0x0705

#endif	/* _FLUKE_EXCEPTION_H_ */
