/*
 * Copyright (c) 1995, 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Glue for the Flick's Fluke backend.  (Flick := Flexible IDL Compiler Kit.)
 * This file is included from every header file generated by that backend.
 * It provides standard CORBA and MOM types and routines.
 */
#ifndef _FLUKE_FLICK_MOM_H_
#define _FLUKE_FLICK_MOM_H_

#include <oskit/machine/types.h>
#include <fluke/ref.h>

/*
 * This opaque structure is what every MOM/CORBA reference points to.
 * It's defined elsewhere, mom_ref.h, in order to avoid namespace pollution,
 * and also because normal MOM programs shouldn't know about its contents.
 */
struct __mom_ref;

/*
 * Standard CORBA types
 */
typedef oskit_s16_t		CORBA_short;
typedef oskit_s32_t		CORBA_long;
typedef oskit_s64_t		CORBA_long_long;
typedef oskit_u16_t		CORBA_unsigned_short;
typedef oskit_u32_t		CORBA_unsigned_long;
typedef oskit_u64_t		CORBA_unsigned_long_long;
typedef float			CORBA_float;
typedef double			CORBA_double;
typedef long double/*XXX?*/	CORBA_long_double;
typedef CORBA_unsigned_long	CORBA_enum;
typedef char			CORBA_char;
typedef CORBA_short		CORBA_wchar;
typedef CORBA_char *		CORBA_string;
typedef CORBA_wchar *		CORBA_wstring;
typedef unsigned char		CORBA_boolean;
typedef unsigned char		CORBA_octet;
typedef struct __flick_ref *	CORBA_Object;

#define CORBA_OBJECT_NIL	((CORBA_Object)0)

typedef enum CORBA_exception_type
{
	CORBA_NO_EXCEPTION,
	CORBA_USER_EXCEPTION,
	CORBA_SYSTEM_EXCEPTION,
} CORBA_exception_type;

typedef enum CORBA_completion_status
{
	CORBA_COMPLETED_YES,
	CORBA_COMPLETED_NO,
	CORBA_COMPLETED_MAYBE
} CORBA_completion_status;

typedef struct CORBA_Environment
{
	CORBA_exception_type	_major;
} CORBA_Environment;

typedef struct CORBA_ex_body
{
	CORBA_unsigned_long	minor;
	CORBA_completion_status	completed;
} CORBA_ex_body;

/* XXX do these belong here? */
typedef CORBA_Object	CORBA_InterfaceDef;
typedef CORBA_Object	CORBA_ImplementationDef;
typedef CORBA_Object	CORBA_Principal;

/*
 * Public MOM types defined by the MOM specification.
 * XXX get rid of these and just use CORBA's?
 */
typedef signed char		mom_char8_t;
typedef oskit_s16_t		mom_char16_t;
typedef oskit_u8_t		mom_unsigned8_t;
typedef oskit_u16_t		mom_unsigned16_t;
typedef oskit_u32_t		mom_unsigned32_t;
typedef oskit_s8_t		mom_signed8_t;
typedef oskit_s16_t		mom_signed16_t;
typedef oskit_s32_t		mom_signed32_t;

typedef struct __flick_ref	*mom_ref_t;
typedef struct __flick_obj	*mom_obj_t;
typedef struct __flick_server	*mom_server_t;

typedef int			mom_key_t;

#if 0
struct mom_obj
{
	void *handle;

	int port_count;
	mach_port_t port[0];
};

#define _MOM_LABEL_BITS 8
#define _MOM_MAX_LABELS	(1 << MOM__LABEL_BITS)

#endif 0

#endif /* _FLUKE_FLICK_MOM_H_ */
