/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_MAPPING_H_
#define _FLUKE_MAPPING_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>
#include <fluke/vm.h>


FLUKE_OB_DEFINE(fluke_mapping, FLUKE_MAPPING_SIZE);

struct fluke_mapping_state
{
	fluke_hash_t	hash;		/* object hash */
	oskit_addr_t	region_off;	/* offset into the region */
	oskit_addr_t	start_va;	/* VA at which the region is mapped */
	oskit_size_t	size;		/* size of mapping */
	fluke_prot_t	prot;		/* protection attributes granted */

	/* Leave at least two pad words for pickled task and region refs.  */
};
typedef struct fluke_mapping_state fluke_mapping_state_t;


struct fluke_ref;

fluke_error_t fluke_mapping_create(
	fluke_mapping_t			*mapping);

fluke_error_t fluke_mapping_create_hash(
	fluke_mapping_t			*mapping,
	fluke_hash_t			*hash);

void fluke_mapping_destroy(
	fluke_mapping_t			*mapping);

void fluke_mapping_protect(
	fluke_mapping_t			*mapping,
	fluke_prot_t			prot);

void fluke_mapping_get_state(
	struct fluke_mapping		*mapping,
	struct fluke_mapping_state	*out_state,
	struct fluke_ref		*out_region_ref,
	struct fluke_ref		*out_task_ref);

void fluke_mapping_set_state(
	struct fluke_mapping		*mapping,
	struct fluke_mapping_state	*state,
	struct fluke_ref		*region_ref,
	struct fluke_ref		*task_ref);

void fluke_mapping_move(
	struct fluke_mapping		*old_va,
	struct fluke_mapping		*new_va);


#include <fluke/machine/mapping.h>

#endif /* _FLUKE_MAPPING_H_ */
