/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Fluke object aware memory manipulation routines
 */
#include <stdlib.h>
#include <fluke/ob.h>

struct floblist {
	struct floblist *next;
	void		*ob;
	fluke_type_t	type;
};

/*
 * Copy [from - from+size] to [to - to+size].  If copyflobs is non-zero,
 * all flobs encountered are copied but left uninitialized.  Otherwise,
 * raw memory is left in the destination.  If flist is non-zero, descriptors
 * for any flobs found in the source are returned.
 *
 * XXX disposition of existing flobs in the destination?
 * XXX what about overlapping ranges?
 * XXX error returns?
 */
int flukemem_copy(void *to, void *from, size_t size, int copyflobs,
		  struct floblist **flist);

/*
 * Removes all flobs from [addr - addr+size].  If keepflobs is non-zero,
 * the flobs are moved to private memory, otherwise they are destroyed.
 * If list is non-zero, descriptors for any moved flobs are returned.
 * XXX need offsets from addr of objects so they can be put back later?
 */
int flukemem_clean(void *addr, size_t size, int keepflobs,
		   struct floblist **flist);

/* 
 * Returns descriptors for all flobs found in [addr - addr+size].
 */
int flukemem_info(void *from, size_t size, struct floblist **flist);

/*
 * Free elements of a floblist.  If keepflobs is non-zero, only the
 * flob list elements are freed.  If zero, all flobs in the list are
 * destroyed and their memory freed as well.
 */
void flukemem_free_floblist(struct floblist *list, int keepflobs);
