/*
 * Copyright (c) 1996, 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This contains all the routines, types, and structures necessary for
 * default handling of the CP routines.
 */
#ifndef _NEST_SERVER_SERVER_H_
#define _NEST_SERVER_SERVER_H_

#include <fluke/ipc.h>
#include <fluke/task.h>
#include <fluke/nest/server/parent.h>
#include <fluke/nest/spawn.h>

#include <mom/ref.h>
#include <mom/object.h>

/*
 * Data structure used by the libnest default Parent functions
 * to describe a nested child task.
 */
struct nest_nester_struct {
	mom_obj_header_t header;
	fluke_task_t *task;	/* fluke_task_destroy'd on exit */
	const char *program;	/* used in messages on exit/exception */
	nest_backing_t *backing_list; /* freed on first parent rpc */
};
#include <fluke/nest/server/nester.h>

/*
 * Create a `nester_child_class' object in `nest_module' (see below)
 * and save `child_task' and `program' in it.
 * The returned object should be used as the first argument to `cp_start_app'.
 */
nest_nester_t *cp_default_pserver (fluke_task_t *child_task,
				   const char *program);

/*
 * Start a new child task by calling spawn;
 * `pserver' is a MOM object to be the parent port (keeper port too).
 */

#if FLASK

int cp_start_app_secure(nest_nester_t *pserver,
			fluke_task_t *child_task,
			const char *program,
			char **argv,
			char **envp,
			security_id_t sid);

#define cp_start_app(pserver,task,program,argv,envp) \
        cp_start_app_secure((pserver),(task),(program),(argv),(envp),SECSID_NULL)

#else

int cp_start_app(nest_nester_t *pserver,
		 fluke_task_t *child_task,
		 const char *program,
		 char **argv,
		 char **envp);

#endif


/*
 * MOM module used by `cp_start_app' for the nester_child object
 * that is the child's Parent port.  Use this module for your own
 * MOM classes too unless you are doing something really fancy.
 */
extern mom_module_t nest_module;

/*
 * MOM class (in `nest_module') for the default `struct nester_child' objects
 * created by `cp_start_app'.  The dispatcher function for the class is
 * `fluke_nester_server', generated from libnest/nester.idl; the
 * cleanup function for the class calls fluke_task_destroy.
 */
extern mom_class_t nester_child_class;

#if FLASK

#define MOMCHECK(_sclass,_ssid,_tclass,_tsid,_perm,_ev) {		      \
        int rc = nest_server_avc_has_permission(SECCLASS_##_sclass, 	      \
					        _ssid,			      \
					        SECCLASS_##_tclass,	      \
					        _tsid,		              \
				    ##_sclass##_##_tclass##__##_perm);	      \
        if (rc) mom_set_exc(_ev, rc);				      \
}

#define MOMCHECKOBJ(_csid,_class,_obj,_perm,_ev) {			      \
        security_id_t objsid; 					              \
	mom_obj_sid((void*)_obj, &objsid);                                    \
        MOMCHECK(SUBJECT,_csid,_class,objsid,_perm,_ev);                      \
}

#define MOMCHECKOBJRET(_csid,_class,_obj,_perm,_ev,_ret) {		      \
        MOMCHECKOBJ(_csid,_class,_obj,_perm,_ev);			      \
        if (_ev->_type) return _ret;					      \
}


extern void child_context(char *appname,
                   	  mandatory_context_t *out_mcontext,
                   	  auth_context_t *out_acontext);

#endif

#endif /* _NEST_CP_SERVER_H_ */
