/*
 * Copyright (c) 1995, 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_NEST_SPAWN_H
#define _FLUKE_NEST_SPAWN_H

#include <fluke/vm.h>
#include <fluke/region.h>
#include <fluke/mapping.h>
#include <fluke/ref.h>
#include <fluke/task.h>
#include <fluke/cp/startup.h>
#include <fluke/port.h>

/*
 * The "backing" for a nested task.  This is a linked list
 * of the region's allocated for an executable.  (The mappings
 * are stored on the nested task's stack, and can be accessed
 * through its' startup.)  The segment_ref must be held to keep
 * the segment alive until the child has made its first rpc back
 * to the parent.
 */
struct nest_backing
{
	mom_ref_t		segment_ref; /* ref for segment io object */
	struct cp_startup_seg	*segment; /* segment info passed to child */
	oskit_addr_t		dest_va;  /* where mem is mapped in child */
	oskit_size_t		size;	  /* current size of segment */
	oskit_size_t		maxsize;  /* maximum size of segment */
	fluke_prot_t		prot;	  /* protection of segment in child */
	void			*rawmem;  /* pointer to mem in parent task */
	struct fluke_mapping	*mapping; /* pointer to mapping for rawmem */
	struct nest_backing	*next;	  /* next struct in list */
};
typedef struct nest_backing nest_backing_t;

/*
 * nest_task
 *
 * returns list of exec_backing_t structures, which contain
 * pointers to all of the regions, etc.
 *
 * XXX Yes, it needs to be pared down a bit.
 */
#if FLASK
#include <mom/ref.h>
#endif

int
#if FLASK
spawn_secure_bootstrap(
#else
spawn(
#endif
	const char	*program,
	int             exec_fd,                /* A filehandle to program */
	char		**argv,
	char		**env,
	fluke_ref_t	*parent_task_ref,
	fluke_ref_t	*scheduler_ref,
	mom_ref_t	parent_ref,
	fluke_ref_t    *keeper_ref,        /* in  */
	fluke_prot_t	user_prot,
	nest_backing_t	**backing_list,		/* out */
	struct cp_startup **thread_startup,	/* out */
#if FLASK
	fluke_task_t	*child_task,
	security_id_t 	sid,
        fluke_ref_t 	*bootstrap);
#else
	fluke_task_t	*child_task);
#endif

#if FLASK
#define spawn_secure(a,b,c,d,e,f,g,h,i,j,k,l) \
        spawn_secure_bootstrap((a),(b),(c),(d),(e),(f),(g),(h),(i),(j),(k),(l),0)

#define spawn(a,b,c,d,e,f,g,h,i,j,k,l) \
        spawn_secure_bootstrap((a),(b),(c),(d),(e),(f),(g),(h),(i),(j),(k),(l),SECSID_NULL,0)
#endif

/*
 * nest_free_backing_list
 *
 * This destroys all of the regions, and mappings, and
 * frees the memory associated with the executable image
 * of a task. (ie it doesn't handle the heap).  It assumes
 * the list passed in is a list returned from nest_task.
 */
void nest_free_backing_list(nest_backing_t *list);

#if FLASK
int spawn_task_create_secure(fluke_task_t **taskpp, security_id_t sid);
#else
int spawn_task_create(fluke_task_t **taskpp);
#endif
void spawn_task_destroy(fluke_task_t *task);

#if FLASK
#define spawn_task_create(tp) \
	spawn_task_create_secure((tp), __mysid)
#endif

#endif


