/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_PORT_H_
#define _FLUKE_PORT_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>

FLUKE_OB_DEFINE(fluke_port, FLUKE_PORT_SIZE);

struct fluke_port_state
{
	fluke_hash_t		hash;

	/* Magic value to pass to service threads upon connection.  */
	oskit_addr_t		alias;

	/* Leave at least 1 pad word for pset reference */
};
typedef struct fluke_port_state fluke_port_state_t;

struct fluke_ref;
struct fluke_pset;

/* Create a new port in the current task.
   The specified port set must already be active in the current task.  */
fluke_error_t fluke_port_create(
	fluke_port_t		*port);

fluke_error_t fluke_port_create_hash(
	fluke_port_t		*port,
	fluke_hash_t		*hash);

void fluke_port_destroy(
	fluke_port_t		*port);

void fluke_port_reference(
	fluke_port_t		*port,
	struct fluke_ref	*out_port_ref);

void fluke_port_move(
	struct fluke_port	*old_va,
	struct fluke_port	*new_va);

void fluke_port_get_state(
	fluke_port_t		*port,
	fluke_port_state_t	*out_state,
	struct fluke_ref	*out_pset_ref);

void fluke_port_set_state(
	fluke_port_t		*port,
	fluke_port_state_t	*state,
	struct fluke_ref	*pset_ref);

#include <fluke/machine/port.h>

#endif /* _FLUKE_PORT_H_ */
