/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_REF_H_
#define _FLUKE_REF_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>


FLUKE_OB_DEFINE(fluke_ref, FLUKE_REF_SIZE);


/* Create a null reference.  */
fluke_error_t fluke_ref_create(
	fluke_ref_t	*ref);

/* Destroy a reference.  */
void fluke_ref_destroy(
	fluke_ref_t	*ref);

/* Make tref reference the same object as fref */ 
void fluke_ref_copy(
	fluke_ref_t	*fref,
	fluke_ref_t	*tref);

/* Compare two references to see if they point to the same Fluke object.  */
int fluke_ref_compare(
	fluke_ref_t	*ref1,
	fluke_ref_t	*ref2);

/* Returns true if the reference is non-null,
   false if the reference is null (invalid).  */
int fluke_ref_check(
	fluke_ref_t	*ref);

/* Move a reference object from one virtual address to another.  */
void fluke_ref_move(
	fluke_ref_t	*old_va,
	fluke_ref_t	*new_va);

/* Return the type of the object that a reference points to.  */
fluke_type_t fluke_ref_type(
	fluke_ref_t	*ref);

/* Return the hash value of the object that a references points to.  */
fluke_hash_t fluke_ref_hash(
	fluke_ref_t	*ref);

#if FLASK
#include <flask/security.h>

/* 
 * Return the security identifier of the object
 * to which the specified reference is linked.
 */ 
security_id_t fluke_ref_sid(fluke_ref_t *ref);

/*
 * Link ref to the security policy server's port.
 *
 * XXX doesn't belong here.
 */
void fluke_security_reference(fluke_ref_t *ref);
#endif

#include <fluke/machine/ref.h>

#endif /* _FLUKE_REF_H_ */
