/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_REGION_H_
#define _FLUKE_REGION_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>
#include <fluke/vm.h>


FLUKE_OB_DEFINE(fluke_region, FLUKE_REGION_SIZE);

struct fluke_region_state
{
	fluke_hash_t	hash;		/* object hash */
	oskit_addr_t	start_va;	/* VA at which the region is mapped */
	oskit_size_t	size;		/* size of region */
	fluke_prot_t	prot;		/* maximum protection granted */

	/* Leave at least two pad words for pickled task and keeper refs.  */
};
typedef struct fluke_region_state fluke_region_state_t;

struct fluke_ref;

fluke_error_t fluke_region_create(
	fluke_region_t		*region);

fluke_error_t fluke_region_create_hash(
	fluke_region_t		*region,
	fluke_hash_t		*hash);

void fluke_region_destroy(
	fluke_region_t		*region);

void fluke_region_protect(
	fluke_region_t		*region,
	fluke_prot_t		prot);

void fluke_region_reference(
	fluke_region_t		*region,
	struct fluke_ref	*region_ref);

void fluke_region_get_state(
	fluke_region_t			*region,
	struct fluke_region_state	*out_state,
	struct fluke_ref		*out_task_ref,
	struct fluke_ref		*out_keeper_ref);

void fluke_region_set_state(
	fluke_region_t			*region,
	struct fluke_region_state	*state,
	struct fluke_ref		*task_ref,
	struct fluke_ref		*keeper_ref);

void fluke_region_move(
	fluke_region_t		*old_va,
	fluke_region_t		*new_va);

/*
 * Search (subset of) a region for kernel objects.
 * If provided array is not large enough for all objects in the range,
 * address will be adjusted to reflect where the search should continue.
 */
void fluke_region_search(
	struct fluke_region		*region,
	oskit_addr_t			*offset,
	oskit_size_t			size,
	struct fluke_obspec		*obarray,
	unsigned int			*obarray_size);

#include <fluke/machine/region.h>

#endif /* _FLUKE_REGION_H_ */
