/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 *	functions to implement POSIX mutexes based on Fluke mutexes
 */

#ifndef _FLUKE_SAC_PTHREAD_MUTEX_H
#define _FLUKE_SAC_PTHREAD_MUTEX_H

/*
 * what we support
 */
#define _POSIX_THREAD_PROCESS_SHARED
#define _POSIX_THREAD_PRIO_INHERIT

/*
 * what we don't support yet
#define _POSIX_THREAD_PRIO_PROTECT
 */

#define PTHREAD_PROCESS_SHARED  0x1
#define PTHREAD_PROCESS_PRIVATE 0x2
#define PTHREAD_PRIO_NONE       0x4
#define PTHREAD_PRIO_INHERIT    0x8
#define PTHREAD_PRIO_PROTECT    0x10

typedef struct pthread_mutexattr {
	long			flags;
} pthread_mutexattr_t;

typedef struct pthread_mutex {
	fluke_mutex_t		*fmutex;	
	pthread_mutexattr_t	attr;
} pthread_mutex_t;

#define PTHREAD_MUTEX_INITIALIZER { 0 }

int 	pthread_mutexattr_init (pthread_mutexattr_t *attr);
int 	pthread_mutexattr_destroy (pthread_mutexattr_t *attr);
int 	pthread_mutexattr_getpshared 
	(const pthread_mutexattr_t *attr, int *pshared);
int 	pthread_mutexattr_setpshared (pthread_mutexattr_t *attr, int pshared);

int 	pthread_mutexattr_setprotocol
	(pthread_mutexattr_t *attr, int protocol);
int 	pthread_mutexattr_getprotocol
	(const pthread_mutexattr_t *attr, int *protocol);
int 	pthread_mutexattr_setprioceiling
	(pthread_mutexattr_t *attr, int prioceiling);
int 	pthread_mutexattr_getprioceiling
	(const pthread_mutexattr_t *attr, int *prioceiling);

int     pthread_mutex_init (pthread_mutex_t *, const pthread_mutexattr_t *);
int     pthread_mutex_lock (pthread_mutex_t *);
int     pthread_mutex_unlock (pthread_mutex_t *);
int     pthread_mutex_trylock (pthread_mutex_t *);
int     pthread_mutex_destroy (pthread_mutex_t *);

#endif /* _FLUKE_SAC_PTHREAD_MUTEX_H */
