/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/* 
 *	pthread_attr.h	- manipulating pthread attributes structures
 */

#ifndef _FLUKE_SAC_PTHREAD_PTHREAD_ATTR_H
#define _FLUKE_SAC_PTHREAD_PTHREAD_ATTR_H

#define _POSIX_THREAD_ATTR_STACKSIZE

/* XXX */
#define PTHREAD_DEFAULT_PRIORITY	0

#define PTHREAD_STACK_DEFAULT           (64*1024)
#define PTHREAD_STACK_MIN           	4096

/* flags */
#define PTHREAD_DETACHED                0x1
#define PTHREAD_SCOPE_SYSTEM            0x2
#define PTHREAD_INHERIT_SCHED           0x4
#define PTHREAD_NOFLOAT                 0x8

#define PTHREAD_CREATE_DETACHED         PTHREAD_DETACHED
#define PTHREAD_CREATE_JOINABLE         0
#define PTHREAD_SCOPE_PROCESS           0
#define PTHREAD_EXPLICIT_SCHED          0

enum schedparam_policy {
        SCHED_RR,
        SCHED_IO,
        SCHED_FIFO,
        SCHED_OTHER
};

struct pthread_attr {
        enum schedparam_policy	schedparam_policy;
        int			prio;

        int			flags;
        void 			*arg_attr;
        void  			(*cleanup_attr)();
        void 			*stackaddr_attr;
        size_t			stacksize_attr;
};

struct sched_param {
	int 			prio;
	void 			*no_data;
};


typedef struct pthread_attr pthread_attr_t;

int     pthread_attr_init               (pthread_attr_t *);
int     pthread_attr_destroy            (pthread_attr_t *);
int     pthread_attr_setstacksize       (pthread_attr_t *, size_t);
int     pthread_attr_getstacksize       (pthread_attr_t *, size_t *);
int     pthread_attr_setstackaddr       (pthread_attr_t *, void *);
int     pthread_attr_getstackaddr       (pthread_attr_t *, void **);
int     pthread_attr_setdetachstate     (pthread_attr_t *, int );
int     pthread_attr_getdetachstate     (pthread_attr_t *, int *);
int     pthread_attr_setcleanup         
	(pthread_attr_t *, void (*routine)(void *), void *);

/* stuff below later */
int     pthread_attr_setscope           (pthread_attr_t *, int );
int     pthread_attr_getscope           (pthread_attr_t *, int *);
int     pthread_attr_setinheritsched    (pthread_attr_t *, int );
int     pthread_attr_getinheritsched 	(pthread_attr_t *, int *);
int     pthread_attr_setschedpolicy 	(pthread_attr_t *, int );
int     pthread_attr_getschedpolicy 	(pthread_attr_t *, int *);
int     pthread_attr_setschedparam      
	(pthread_attr_t *, struct sched_param *);
int     pthread_attr_getschedparam      
	(pthread_attr_t *, struct sched_param *);

int     pthread_attr_setfloatstate      (pthread_attr_t *, int );
int     pthread_attr_getfloatstate      (pthread_attr_t *, int *);

#endif	/* _FLUKE_SAC_PTHREAD_PTHREAD_ATTR_H */
