/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_TASK_H_
#define _FLUKE_TASK_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>


FLUKE_OB_DEFINE(fluke_task, FLUKE_TASK_SIZE);

struct fluke_ref;

struct fluke_task_state
{
	fluke_hash_t	hash;
};
typedef struct fluke_task_state fluke_task_state_t;

/* Create a new empty task, initially devoid of any mappings or threads.  */
fluke_error_t fluke_task_create(
	fluke_task_t		*task);

fluke_error_t fluke_task_create_hash(
	fluke_task_t		*task,
	fluke_hash_t		*hash);

/* Destroy a task.  All threads in the task are disabled.  */
void fluke_task_destroy(
	fluke_task_t		*task);

void fluke_task_reference(
	struct fluke_task		*task,
	struct fluke_ref		*out_task_ref);

void fluke_task_move(
	struct fluke_task		*old_va,
	struct fluke_task		*new_va);

void fluke_task_get_state(
	struct fluke_task		*task,
	struct fluke_task_state		*out_state,
	struct fluke_ref		*out_keeper_ref);

void fluke_task_set_state(
	struct fluke_task		*task,
	struct fluke_task_state		*state,
	struct fluke_ref		*keeper_ref);


#include <fluke/machine/task.h>

#endif /* _FLUKE_TASK_H_ */
