/*
 * Copyright (c) 1995, 1996, 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_THREAD_H_
#define _FLUKE_THREAD_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>

#if FLASK
#include <flask/security.h>
#endif

FLUKE_OB_DEFINE(fluke_thread, FLUKE_THREAD_SIZE);

struct fluke_thread_state;
#if FLASK
struct fluke_thread_state_secure;
#endif
struct fluke_thread_saved_state;
struct fluke_ref;
struct fluke_pset;

/* Create a disabled, uninitialized thread.  */
fluke_error_t fluke_thread_create(
	fluke_thread_t			*thread);

fluke_error_t fluke_thread_create_hash(
	fluke_thread_t			*thread,
	fluke_hash_t			*hash);

/* Terminate an arbitrary thread.  */
void fluke_thread_destroy(
	fluke_thread_t			*thread);

void fluke_thread_reference(
	fluke_thread_t			*thread,
	struct fluke_ref		*out_thread_ref);

void fluke_thread_move(
	fluke_thread_t			*old_va,
	fluke_thread_t			*new_va);

void fluke_thread_get_client(
	fluke_thread_t			*cur_thread,
	struct fluke_ref		*client_ref,
	unsigned			*flags);

#if FLASK
void fluke_thread_get_client_secure(
	fluke_thread_t			*cur_thread,
	struct fluke_ref		*client_ref,
        security_id_t			*effective_server_sid, 
        security_id_t			*client_effective_client_sid, 
	unsigned			*flags);
#endif

void fluke_thread_set_client(
	fluke_thread_t			*cur_thread,
	struct fluke_ref		*client_ref,
	unsigned			flags);

#if FLASK
void fluke_thread_set_client_secure(
	fluke_thread_t			*cur_thread,
	struct fluke_ref		*client_ref,
        security_id_t			effective_server_sid,
        security_id_t			client_effective_client_sid,
	unsigned			flags);
#endif

void fluke_thread_get_server(
	fluke_thread_t			*cur_thread,
	struct fluke_ref		*server_ref,
	unsigned			*flags);

#if FLASK
void fluke_thread_get_server_secure(
	fluke_thread_t			*cur_thread,
	struct fluke_ref		*server_ref,
        security_id_t			*effective_client_sid,
        security_id_t			*server_effective_server_sid,
	unsigned			*flags);
#endif

void fluke_thread_set_server(
	fluke_thread_t			*cur_thread,
	struct fluke_ref		*server_ref,
	unsigned			flags);

#if FLASK
void fluke_thread_set_server_secure(
	fluke_thread_t			*cur_thread,
	struct fluke_ref		*server_ref,
        security_id_t			effective_client_sid, 
        security_id_t			server_effective_server_sid,
	unsigned			flags);
#endif 

void fluke_thread_get_state(
	fluke_thread_t			*thread,
	struct fluke_thread_state	*out_state,
	struct fluke_ref		*out_task_ref,
	struct fluke_ref		*out_scheduler_ref,
	struct fluke_ref		*out_waiting_for_ref,
	struct fluke_ref		*out_client_ref,
	struct fluke_ref		*out_server_ref);

#if FLASK
void fluke_thread_get_state_secure(
	fluke_thread_t			*thread,
	struct fluke_thread_state_secure *out_state,
	struct fluke_ref		*out_task_ref,
	struct fluke_ref		*out_scheduler_ref,
	struct fluke_ref		*out_waiting_for_ref,
	struct fluke_ref		*out_client_ref,
	struct fluke_ref		*out_server_ref);
#endif

void fluke_thread_set_state(
	fluke_thread_t			*thread,
	struct fluke_thread_state	*state,
	struct fluke_ref		*task_ref,
	struct fluke_ref		*scheduler_ref,
	struct fluke_ref		*waiting_for_ref,
	struct fluke_ref		*client_ref,
	struct fluke_ref		*server_ref);

#if FLASK
void fluke_thread_set_state_secure(
	fluke_thread_t			*thread,
	struct fluke_thread_state_secure *state,
	struct fluke_ref		*task_ref,
	struct fluke_ref		*scheduler_ref,
	struct fluke_ref		*waiting_for_ref,
	struct fluke_ref		*client_ref,
	struct fluke_ref		*server_ref);
#endif

/*
 * Change the current thread's exception enable flag (E flag),
 * which allows this thread to handle its own exceptions.
 */
void fluke_thread_enable_exceptions(
	fluke_thread_t			*thread);

void fluke_thread_disable_exceptions(
	fluke_thread_t			*thread);


/* Post an asynchronous software interrupt to a thread.  */
void fluke_thread_interrupt(
	fluke_thread_t			*thread);

/* Find the current thread */
fluke_thread_t *fluke_thread_self(void);

/*
 * Get and set the current thread's saved exception handler registers.
 * Normally only called by exception handlers.
 * 'cur_thread' must be the current thread,
 * as returned by fluke_thread_self()
 * (or as provided to the exception handler on entry).
 */
void fluke_thread_get_saved_state(
	struct fluke_thread		*cur_thread,
	struct fluke_thread_saved_state	*out_saved_state);

void fluke_thread_set_saved_state(
	struct fluke_thread		*cur_thread,
	struct fluke_thread_saved_state	*saved_state);

/*
 * Get/Set the exception handlers for the current thread.
 */
void fluke_thread_get_handlers(
	struct fluke_thread		*cur_thread,
	oskit_addr_t			*trap_handler,
	oskit_addr_t			*interrupt_handler,
	oskit_addr_t			*client_alert_handler,
	oskit_addr_t			*server_alert_handler);

void fluke_thread_set_handlers(
	struct fluke_thread		*cur_thread,
	oskit_addr_t			trap_handler,
	oskit_addr_t			interrupt_handler,
	oskit_addr_t			client_alert_handler,
	oskit_addr_t			server_alert_handler);

/*
 * Donate time to the traget thread, or if NULL, any runnable thread.
 */
void fluke_thread_yield(
	struct fluke_thread		*cur_thread,
	struct fluke_thread		*target);

#include <fluke/machine/thread.h>

#endif /* _FLUKE_THREAD_H_ */
