/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_X86_COND_H_
#define _FLUKE_X86_COND_H_

#if defined(__GNUC__) && defined(FLUKE_INLINING)

#include <oskit/compiler.h>
#ifndef FLUKE_ENTRY_INLINE
#define FLUKE_ENTRY_INLINE OSKIT_INLINE
#endif
#ifdef GPROF
#define e(name) extern char _fluke_##name##_[];
#include <fluke/x86/syscall_list.h>
#undef e
#include <oskit/x86/proc_reg.h>
#include <fluke/profile.h>
#endif

FLUKE_ENTRY_INLINE fluke_error_t fluke_cond_create(
	fluke_cond_t		*cond)
{
	fluke_error_t rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_create_hash_);
#endif
	asm volatile("call	_fluke_cond_create_hash"
		: "=a" (rc) : "D" (cond), "d" (0) : "eax", "ecx", "edx");

	return rc;
}

FLUKE_ENTRY_INLINE fluke_error_t fluke_cond_create_hash(
	fluke_cond_t	*cond,
	fluke_hash_t	*hashp)
{
	fluke_error_t rc;
	fluke_hash_t hash;

	hash = hashp ? *hashp : 0;
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_create_hash_);
#endif
	asm volatile("call _fluke_cond_create_hash"
		: "=a" (rc), "=d" (hash)
		: "D" (cond), "d" (hash)
		: "ecx");
	if (hashp) *hashp = hash;

	return rc;
}

FLUKE_ENTRY_INLINE void fluke_cond_destroy(
	fluke_cond_t		*cond)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_destroy_);
#endif
	asm volatile("call	_fluke_cond_destroy"
		: : "D" (cond) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_cond_wait(
	fluke_cond_t		*cond,
	struct fluke_mutex	*mutex)
{
	/*
	 * Note: the "memory" here is necessary,
	 * not because this particular system call
	 * modifies any memory GCC cares about (it doesn't),
	 * but because other threads will run while we are waiting
	 * and presumably make changes to memory
	 * that we need to know about.
	 */
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_wait_);
#endif
	asm volatile("call	_fluke_cond_wait"
		: : "D" (cond), "S" (mutex) : "eax", "ecx", "edx", "memory");
}

FLUKE_ENTRY_INLINE void fluke_cond_signal(
	fluke_cond_t		*cond)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_signal_);
#endif
	asm volatile("call	_fluke_cond_signal"
		: : "D" (cond) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_cond_broadcast(
	fluke_cond_t		*cond)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_broadcast_);
#endif
	asm volatile("call	_fluke_cond_broadcast"
		: : "D" (cond) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_cond_reference(
	fluke_cond_t		*cond,
	struct fluke_ref	*out_cond_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_reference_);
#endif
	asm volatile("call	_fluke_cond_reference"
		: : "a" (cond), "d" (out_cond_ref) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_cond_move(
	fluke_cond_t		*old_va,
	fluke_cond_t		*new_va)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_move_);
#endif
	asm volatile("call	_fluke_cond_move"
		: : "a" (old_va), "d" (new_va) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_cond_get_state(
	fluke_cond_t		*cond,
	fluke_cond_state_t	*out_state,
	struct fluke_ref	*out_inheritor_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_get_state_);
#endif
	asm volatile("call	_fluke_cond_get_state"
		: : "a" (cond), "d" (out_state), "c" (out_inheritor_ref)
		: "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_cond_set_state(
	fluke_cond_t		*cond,
	fluke_cond_state_t	*state,
	struct fluke_ref	*inheritor_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_cond_set_state_);
#endif
	asm volatile("call	_fluke_cond_set_state"
		: : "a" (cond), "d" (state), "c" (inheritor_ref)
		: "eax", "ecx", "edx");
}

#endif /* __GNUC__ */

#endif /* _FLUKE_X86_COND_H_ */
