/*
 * Copyright (c) 1995, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Debug functions. 
 */
#ifndef FLUKE_X86_DEBUG_H
#define FLUKE_X86_DEBUG_H

#if defined(__GNUC__) && defined(FLUKE_INLINING)

#include <oskit/compiler.h>
#ifndef FLUKE_ENTRY_INLINE
#define FLUKE_ENTRY_INLINE OSKIT_INLINE
#endif
#include <string.h>
#ifdef GPROF
#define e(name) extern char _fluke_##name##_[];
#include <fluke/x86/syscall_list.h>
#undef e
#include <oskit/x86/proc_reg.h>
#include <fluke/profile.h>
#endif

FLUKE_ENTRY_INLINE void fluke_debug_dump_flob(
	void 	*sob)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_debug_dump_flob_);
#endif
	asm volatile("call _fluke_debug_dump_flob"
		     : 
		     : "a" (sob)
		     : "eax", "ecx", "edx");
}

/*
 * Perhaps you are wondering why we have this here.  Its
 * because we don't want to mess with copyin_local() in
 * the kernel, which expects polite(word aligned, even length),
 * fully available user mode buffers.  Instead of doing the
 * "correct thing" we did this:
 *
 * XXX this is a security hole
 */
static char __fluke_debug_puts_str_buffer[FLUKE_DEBUG_PUTS_STR_LEN+1];

FLUKE_ENTRY_INLINE void fluke_debug_puts(
	const char 	*str)
{
	int len = strlen(str);

	assert(len < FLUKE_DEBUG_PUTS_STR_LEN);

	strncpy(__fluke_debug_puts_str_buffer, str, len);
	__fluke_debug_puts_str_buffer[len] = '\0';

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_debug_puts_);
#endif
	asm volatile("call _fluke_debug_puts"
		     : 
		     : "a" (__fluke_debug_puts_str_buffer),  "b" (len) 
		     : "eax", "ebx", "ecx", "edx");
}

/*
 * Same reason as above, we only allow 8 character labels in the kernel flobs
 * anyway...
 *
 * XXX this is a monster security hole in the kernel
 */
static char __fluke_debug_label_str_buffer[FLUKE_DEBUG_LABEL_STR_MAXLEN];

FLUKE_ENTRY_INLINE void fluke_debug_label_flob(
	void       *sob,
	const char *label)
{
	int ct = 0;

	assert(sob);
	assert(label);

	/*
	 * Copy into our local, static buffer -- kernel can ``assume''
	 * the buffer is all there.  We also guarantee to the kernel
	 * that the string is NULL terminated.
	 */
	while(*label && (ct < FLUKE_DEBUG_LABEL_STR_MAXLEN - 1))
		__fluke_debug_label_str_buffer[ct++] = *(label++);
	__fluke_debug_label_str_buffer[ct] = '\0';

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_debug_label_flob_);
#endif
	asm volatile("call _fluke_debug_label_flob"
		     :
		     : "a" (sob), "b" (__fluke_debug_label_str_buffer)
		     : "eax", "ebx", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_debug_dump_memuse(void)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_debug_dump_memuse_);
#endif
	asm volatile("call	_fluke_debug_dump_memuse"
		: : : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_debug_null_syscall(void)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_debug_null_syscall_);
#endif
	asm volatile("call	_fluke_debug_null_syscall"
		: : : "eax", "ecx", "edx");
}

OSKIT_INLINE void fluke_debug_set_monregs(
	unsigned	reg0,
	unsigned	reg1)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_debug_set_monregs_);
#endif
	asm volatile("call _fluke_debug_set_monregs"
		     : 
		     : "a" (reg0),  "b" (reg1) 
		     : "eax", "ebx", "ecx", "edx");
}

#endif /* __GNUC__ */

#endif /* FLUKE_X86_DEBUG_H */
