/*
 * Copyright (c) 1995, 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_X86_PSET_H_
#define _FLUKE_X86_PSET_H_

#if defined(__GNUC__) && defined(FLUKE_INLINING)

#include <oskit/compiler.h>
#ifndef FLUKE_ENTRY_INLINE
#define FLUKE_ENTRY_INLINE OSKIT_INLINE
#endif
#ifdef GPROF
#define e(name) extern char _fluke_##name##_[];
#include <fluke/x86/syscall_list.h>
#undef e
#include <oskit/x86/proc_reg.h>
#include <fluke/profile.h>
#endif

FLUKE_ENTRY_INLINE fluke_error_t fluke_pset_create(
	fluke_pset_t	*pset)
{
	fluke_error_t rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_pset_create_hash_);
#endif
	asm volatile("call _fluke_pset_create_hash"
		: "=a" (rc) : "a" (pset), "d" (0) : "ecx", "edx");

	return rc;
}

FLUKE_ENTRY_INLINE fluke_error_t fluke_pset_create_hash(
	fluke_pset_t	*pset,
	fluke_hash_t	*hashp)
{
	fluke_error_t rc;
	fluke_hash_t hash;

	hash = hashp ? *hashp : 0;
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_pset_create_hash_);
#endif
	asm volatile("call _fluke_pset_create_hash"
		: "=a" (rc), "=d" (hash)
		: "a" (pset), "d" (hash)
		: "ecx");
	if (hashp) *hashp = hash;

	return rc;
}

FLUKE_ENTRY_INLINE void fluke_pset_destroy(
	fluke_pset_t	*pset)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_pset_destroy_);
#endif
	asm volatile("call _fluke_pset_destroy"
		: : "a" (pset) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_pset_get_state(
	struct fluke_pset	*pset,
	struct fluke_pset_state	*out_state)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_pset_get_state_);
#endif
	asm volatile("call _fluke_pset_get_state"
		: : "a" (pset), "d" (out_state) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_pset_set_state(
	struct fluke_pset	*pset,
	struct fluke_pset_state	*state)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_pset_set_state_);
#endif
	asm volatile("call _fluke_pset_set_state"
		: : "a" (pset), "d" (state) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_pset_move(
	fluke_pset_t	*opset,
	fluke_pset_t	*npset)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_pset_move_);
#endif
	asm volatile("call _fluke_pset_move"
		: : "a" (opset), "d" (npset) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_pset_reference(
	struct fluke_pset		*pset,
	struct fluke_ref		*pset_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_pset_reference_);
#endif
	asm volatile("call _fluke_pset_reference"
		: : "a" (pset), "d" (pset_ref) : "eax", "ecx", "edx");
}

#endif /* __GNUC__ */

#endif /* _FLUKE_X86_PSET_H_ */
