/*
 * Copyright (c) 1995, 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_X86_REF_H_
#define _FLUKE_X86_REF_H_

#if defined(__GNUC__) && defined(FLUKE_INLINING)

#include <oskit/compiler.h>
#ifndef FLUKE_ENTRY_INLINE
#define FLUKE_ENTRY_INLINE OSKIT_INLINE
#endif
#ifdef GPROF
#define e(name) extern char _fluke_##name##_[];
#include <fluke/x86/syscall_list.h>
#undef e
#include <oskit/x86/proc_reg.h>
#include <fluke/profile.h>
#endif

FLUKE_ENTRY_INLINE fluke_error_t fluke_ref_create(
	fluke_ref_t	*ref)
{
	fluke_error_t rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_create_);
#endif  
	asm volatile("call _fluke_ref_create"
		: "=a" (rc) : "a" (ref) : "ecx", "edx");

	return rc;
}

FLUKE_ENTRY_INLINE void fluke_ref_destroy(
	fluke_ref_t	*ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_destroy_);
#endif  
	asm volatile("call _fluke_ref_destroy"
		: : "a" (ref) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_ref_move(
	fluke_ref_t	*oref,
	fluke_ref_t	*nref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_move_);
#endif  
	asm volatile("call _fluke_ref_move"
		: : "a" (oref), "d" (nref) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE int fluke_ref_check(
	fluke_ref_t	*ref)
{
	fluke_error_t rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_check_);
#endif  
	asm volatile("call _fluke_ref_check"
		: "=a" (rc) : "a" (ref) : "ecx");

	return rc;
}

FLUKE_ENTRY_INLINE int fluke_ref_compare(
	fluke_ref_t	*ref1,
	fluke_ref_t	*ref2)
{
	fluke_error_t rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_compare_);
#endif  
	asm volatile("call _fluke_ref_compare"
		: "=a" (rc) : "a" (ref1), "d" (ref2) : "ecx", "edx");

	return rc;
}

FLUKE_ENTRY_INLINE void fluke_ref_copy(
	fluke_ref_t	*fref,
	fluke_ref_t	*tref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_copy_);
#endif  
	asm volatile("call _fluke_ref_copy"
		: : "a" (fref), "d" (tref) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE fluke_type_t
fluke_ref_type(fluke_ref_t *ref)
{
	fluke_type_t type;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_type_);
#endif  
	asm volatile("call _fluke_ref_type" : "=a" (type) : "a" (ref));
	return type;
}

FLUKE_ENTRY_INLINE fluke_hash_t
fluke_ref_hash(fluke_ref_t *ref)
{
	fluke_hash_t hash;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_hash_);
#endif  
	asm volatile("call _fluke_ref_hash" : "=a" (hash) : "a" (ref));
	return hash;
}

#if FLASK
#include <flask/security.h>

FLUKE_ENTRY_INLINE security_id_t
fluke_ref_sid(fluke_ref_t *ref)
{
        security_id_t sid;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_ref_sid_);
#endif  
	asm volatile("call _fluke_ref_sid" 
		     : "=a" (sid) 
		     : "a" (ref));
	return sid;
}


/* XXX:  doesn't belong here */
FLUKE_ENTRY_INLINE void
fluke_security_reference(fluke_ref_t *ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_security_reference_);
#endif  
	asm volatile("call _fluke_security_reference" 
		     : : "a" (ref));
}
#endif

#endif /* __GNUC__ */

#endif /* _FLUKE_X86_REF_H_ */
