/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_X86_REGION_H_
#define _FLUKE_X86_REGION_H_

#if defined(__GNUC__) && defined(FLUKE_INLINING)

#include <oskit/compiler.h>
#ifndef FLUKE_ENTRY_INLINE
#define FLUKE_ENTRY_INLINE OSKIT_INLINE
#endif
#ifdef GPROF
#define e(name) extern char _fluke_##name##_[];
#include <fluke/x86/syscall_list.h>
#undef e
#include <oskit/x86/proc_reg.h>
#include <fluke/profile.h>
#endif

FLUKE_ENTRY_INLINE fluke_error_t fluke_region_create(
	fluke_region_t	*region)
{
	fluke_error_t rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_create_hash_);
#endif
	asm volatile("call _fluke_region_create_hash"
		: "=a" (rc) : "a" (region), "d" (0) : "ecx", "edx");

	return rc;
}

FLUKE_ENTRY_INLINE fluke_error_t fluke_region_create_hash(
	fluke_region_t	*region,
	fluke_hash_t	*hashp)
{
	fluke_error_t rc;
	fluke_hash_t hash;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_create_hash_);
#endif
	hash = hashp ? *hashp : 0;

	asm volatile("call _fluke_region_create_hash"
		: "=a" (rc), "=d" (hash)
		: "a" (region), "d" (hash)
		: "ecx");
	if (hashp) *hashp = hash;

	return rc;
}

FLUKE_ENTRY_INLINE void fluke_region_destroy(
	fluke_region_t	*region)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_destroy_);
#endif
	asm volatile("call _fluke_region_destroy"
		: : "a" (region) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_region_reference(
	struct fluke_region		*region,
	struct fluke_ref		*region_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_reference_);
#endif
	asm volatile("call _fluke_region_reference"
		: : "a" (region), "d" (region_ref) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_region_protect(
	struct fluke_region		*region,
	fluke_prot_t			prot)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_protect_);
#endif
	asm volatile("call _fluke_region_protect"
		: : "a" (region), "d" (prot) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_region_get_state(
	struct fluke_region		*region,
	struct fluke_region_state	*out_state,
	struct fluke_ref		*out_task_ref,
	struct fluke_ref		*out_keeper_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_get_state_);
#endif
	asm volatile("call _fluke_region_get_state"
		: : "a" (region), "d" (out_state),
		    "c" (out_task_ref), "b" (out_keeper_ref)
		: "eax", "ebx", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_region_set_state(
	struct fluke_region		*region,
	struct fluke_region_state	*state,
	struct fluke_ref		*task_ref,
	struct fluke_ref		*keeper_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_set_state_);
#endif
	asm volatile("call _fluke_region_set_state"
		: : "a" (region), "d" (state),
		    "c" (task_ref), "b" (keeper_ref)
		: "eax", "ebx", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_region_move(
	fluke_region_t	*oregion,
	fluke_region_t	*nregion)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_move_);
#endif
	asm volatile("call _fluke_region_move"
		: : "a" (oregion), "d" (nregion) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_region_search(
	struct fluke_region		*region,
	oskit_addr_t			*offset,	/* in/out */
	oskit_size_t			size,
	struct fluke_obspec		*obarray,
	unsigned int			*obarray_size)	/* in/out */
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_region_search_);
#endif
	asm volatile("call _fluke_region_search"
		: "=d" (*offset), "=S" (*obarray_size)
		: "a" (region), "d" (*offset),
		  "c" (size), "b" (obarray), "S" (*obarray_size)
		: "eax", "ebx", "ecx", "edx", "esi", "memory");
}

#endif /* __GNUC__ */

#endif /* _FLUKE_X86_REGION_H_ */
