/*
 * Copyright (c) 1996 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include "acount.h"
#include "preempt.h"
#include "debug.h"

#ifndef MD_ACOUNT
/*
 * Atomic counters.
 *
 * This is the portable version that uses spin locks for atomicity.
 * An optimized version would use some architecture defined atomic primitive.
 */

void acount_init(
	struct acount *ac,
	int value)
{
	spin_lock_init(&ac->lock);
	ac->counter = value;
}

int acount_incr(
	struct acount *ac)
{
	int rc;

	disable_preemption();
	spin_lock(&ac->lock);
	rc = (ac->counter)++;
	spin_unlock(&ac->lock);
	enable_preemption();

	assert(rc >= 0);
	return (rc);
}

int acount_decr(
	struct acount *ac)
{
	int rc;

	disable_preemption();
	spin_lock(&ac->lock);
	rc = --(ac->counter);
	spin_unlock(&ac->lock);
	enable_preemption();

	assert(rc >= 0);
	return (rc);
}
#endif  /* MD_ACOUNT */
