/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Generic prototypes for machine-dependent context switching code.
 * The 'csw_state' structure must be defined by the MD csw.h header.
 */
#ifndef _FLUKE_KERNEL_CSW_H_
#define _FLUKE_KERNEL_CSW_H_

#include "config_nonblocking.h"

struct s_thread;

/* Boot-time initialization of the context switch layer */
void csw_boot(void);

/* Initialize the per-thread state used by the context switch layer.  */
void thread_csw_state_init(
	struct s_thread		*thread);

/* Start the first thread running on a processor */
void csw_start(
	struct s_thread		*cur_thread);

/* Go into user mode for the first time */
void csw_start_user(
	struct s_thread		*cur_thread);


#ifndef CONFIG_NONBLOCKING

/*
 * In a blocking kernel, this sets up the kernel stack in a new thread
 * so that the first time csw_switch() is called with that thread as the target,
 * the thread will immediately call 'init_func', and when that returns,
 * it will jump to 'start_func' with a stack pointer of 'start_stack_pointer'.
 */
void thread_csw_state_setup_stack(
	struct s_thread		*thread,
	int			(*init_func)(void),
	void			(*start_func)(void),
	void			*start_stack_pointer);

#endif /* not CONFIG_NONBLOCKING */

/*
 * Context switch from the current thread to a different thread.
 * Saves all the necessary current-thread context in cur_thread,
 * and loads the new context from next_thread.
 * In blocking kernels, a stack switch is performed,
 * so the csw_switch() call will not appear to return
 * until some processor switches back to this same thread.
 * In nonblocking kernels, no stack switch is done,
 * so the csw_switch() will return immediately but in a different thread.
 * In any case, csw_switch() returns the new current thread as its result.
 */
struct s_thread *csw_switch(
	struct s_thread		*cur_thread,
	struct s_thread		*next_thread);

#endif /* _FLUKE_KERNEL_CSW_H_ */
