/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Contributed by the Computer Security Research division,
 * INFOSEC Research and Technology Office, NSA.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#if FLASK

#include "frag.h"
#include "s_mapping.h"
#include "debug.h"

#include "avc.h"

#ifdef USEAMM
typedef struct amm_revoke_args {
    security_id_t   tsid;
    access_vector_t revoke;
    oskit_bool_t    try_remove;
    access_vector_t *out_retained;
}               amm_revoke_args_t;


static int
amme_revoke(struct amm * amm, struct amm_entry * amme, void *args)
{
    frag_t         *frag;
    struct amm_revoke_args *a = (struct amm_revoke_args *) args;
    security_id_t   tsid = a->tsid;
    access_vector_t revoke = a->revoke;
    oskit_bool_t    try_remove = a->try_remove;
    access_vector_t *out_retained = a->out_retained;

    if (amm_entry_flags(amme) == AMM_FREE)
	return 0;

    frag = AMMETOFRAG(amme);

    if (frag_segment_sid(frag) == tsid) {
	if (!try_remove)
	    frag_protect(frag, frag_prot(frag) & ~revoke);
	else {
	    *out_retained |= (frag_prot(frag) & revoke);
	    if ((*out_retained & revoke) == revoke)
		/* no need to continue searching */
		return 1;
	}
    }
    return 0;
}
#endif

void
mapping_frag_revoke(struct s_mapping * mp, security_id_t tsid,
		    access_vector_t revoke, oskit_bool_t try_remove,
		    access_vector_t * out_retained)
{
#ifdef USEAMM
    amm_revoke_args_t args;
    args.tsid = tsid;
    args.revoke = revoke;
    args.try_remove = try_remove;
    args.out_retained = out_retained;
    amm_iterate(&mp->fragments, amme_revoke, (void *) &args);
#else
    frag_t         *frag;
    queue_iterate(&mp->fragments, frag, frag_t *, mlink) {
	if (frag_segment_sid(frag) == tsid) {
	    if (!try_remove)
		frag_protect(frag, frag_prot(frag) & ~revoke);
	    else {
		*out_retained |= (frag_prot(frag) & revoke);
		if ((*out_retained & revoke) == revoke)
		    /* no need to continue searching */
		    break;
	    }
	}
    }
#endif
}



#ifdef USEAMM
static int
amme_reset(struct amm * amm, struct amm_entry * amme, void *args)
{
    security_id_t   task_sid = (security_id_t) args;
    struct avc_entry *ae;
    unsigned        aever;
    access_vector_t allowed;
    frag_t         *frag;
    int             rc;


    if (amm_entry_flags(amme) == AMM_FREE)
	return 0;

    frag = AMMETOFRAG(amme);

    rc = avc_compute_access_vector(SECCLASS_SUBJECT,
				   task_sid,
				   SECCLASS_SUBJECT,
				   frag_segment_sid(frag),
				   frag_prot(frag),
				   &allowed, &ae, &aever);
    if (rc)
	return rc;

    if ((frag_prot(frag) & allowed) != frag_prot(frag))
	frag_protect(frag, frag_prot(frag) & allowed);

    return 0;
}
#endif

int
mapping_frag_reset(struct s_mapping * mp, security_id_t task_sid)
{
#ifdef USEAMM
    return amm_iterate(&mp->fragments, amme_reset, (void *) task_sid);
#else
    struct avc_entry ae;
    unsigned        aever;
    access_vector_t allowed;
    frag_t         *frag;
    int             rc;


    queue_iterate(&mp->fragments, frag, frag_t *, mlink) {
	rc = avc_compute_access_vector(SECCLASS_SUBJECT,
				       task_sid,
				       SECCLASS_SUBJECT,
				       frag_segment_sid(frag),
				       frag_prot(frag),
				       &allowed, &ae, &aever);
	if (rc)
	    return rc;

	if ((frag_prot(frag) & allowed) != frag_prot(frag))
	    frag_protect(frag, frag_prot(frag) & allowed);

    }
    return 0;
#endif
}
#endif
