/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Machine-independent definitions for hardware interrupt handling.
 * Machine-dependent code actually implements the routines defined here.
 */
#ifndef _FLUKE_KERNEL_INTR_H_
#define _FLUKE_KERNEL_INTR_H_

/*
 * Attach a normal device interrupt handler to an IRQ line.
 * Normal handlers can potentially share the same interrupt line,
 * but cannot be sensitive to the state
 * of the interrupted user- or kernel-mode activity.
 */
int irq_alloc(int irq, void (*func), void *data, int shareable);

void irq_free(int irq, void *func, void *data);

#if 0
/*
 * Block the current thread waiting on the specified hardware interrupt.
 * Assumes that interrupts are enabled, and returns with interrupts enabled.
 */
int interrupt_wait(unsigned irq);
#endif

/*
 * Initialize the interrupt layer at boot time.
 */
void interrupt_boot(void);

#endif /* _FLUKE_KERNEL_INTR_H_ */
