/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Contributed by the Computer Security Research division,
 * INFOSEC Research and Technology Office, NSA.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#if FLASK

#include "link.h"
#include "ipc.h"
#include "s_thread.h"
#include "debug.h"
#include "avc.h"


/*
 * If the peer exists, it will always be in the link, since the thread has
 * been stopped.  Since all threads have been stopped, we do not need to lock
 * and/or stop the peer thread.
 */
#define GETPEER(thread,peername,peer) \
  *(peer) = (s_thread_t*)thread->ipc_state.peername##_link.target; \
  if ((*(peer)) && \
	(((*(peer))->sob.type != FLUKE_THREAD)  || \
	 !((*(peer))->sob.active))) \
      *(peer) = NULL;


#define CHECKIPC(rcsid,ecsid,rssid,essid) \
         (((revoke & SUBJECT_SUBJECT__SPECIFY_CLIENT) && \
	 ssid != tsid && rcsid == ssid && ecsid == tsid) || \
            ((revoke & SUBJECT_SUBJECT__SPECIFY_SERVER) && \
             ssid != tsid && rssid == ssid && essid == tsid) || \
            ((revoke & SUBJECT_SUBJECT__CONNECT) && \
             ecsid == ssid && essid == tsid))


#define CHECKRETAINED(rcsid,ecsid,rssid,essid) { \
	if ((revoke & SUBJECT_SUBJECT__SPECIFY_CLIENT) && \
            ssid != tsid && rcsid == ssid && ecsid == tsid) \
		retained |= SUBJECT_SUBJECT__SPECIFY_CLIENT; \
	if ((revoke & SUBJECT_SUBJECT__SPECIFY_SERVER) && \
             ssid != tsid && rssid == ssid && essid == tsid) \
		retained |= SUBJECT_SUBJECT__SPECIFY_SERVER; \
	if ((revoke & SUBJECT_SUBJECT__CONNECT) && \
             ecsid == ssid && essid == tsid) \
                retained |= SUBJECT_SUBJECT__CONNECT; \
	}


void
thread_ipc_revoke(struct s_thread * thread,
		  security_id_t ssid,
		  security_id_t tsid,
		  access_vector_t revoke,
		  oskit_bool_t try_remove,
		  access_vector_t * out_retained)
{
    struct s_thread *peer = NULL;
    security_id_t   peer_esid;
    access_vector_t retained = 0;

    /* thread is locked and stopped on entry */

    GETPEER(thread, server, &peer);
    if (peer) {
	if (thread->ipc_state.server_effective_server_sid)
	    peer_esid = thread->ipc_state.server_effective_server_sid;
	else
	    peer_esid = peer->ipc_state.effective_server_sid;

	if (CHECKIPC(thread->sob.sid, thread->ipc_state.effective_client_sid,
		     peer->sob.sid, peer_esid)) {
	    if (!try_remove) {
		thread->ipc_state.effective_client_sid = thread->sob.sid;
		thread->ipc_state.server_effective_server_sid = SECSID_NULL;
		link_reinit(&thread->ipc_state.server_link);
	    } else {
		CHECKRETAINED(thread->sob.sid,
			      thread->ipc_state.effective_client_sid,
			      peer->sob.sid, peer_esid);
	    }
	}
    } else if ((revoke & SUBJECT_SUBJECT__SPECIFY_CLIENT) &&
	       ssid != tsid &&
	       thread->sob.sid == ssid &&
	       thread->ipc_state.effective_client_sid == tsid) {
	if (!try_remove)
	    thread->ipc_state.effective_client_sid = thread->sob.sid;
	else
	    retained |= SUBJECT_SUBJECT__SPECIFY_CLIENT;
    }
    if (try_remove && retained == revoke) {
	*out_retained |= retained;
	return;
    }
    GETPEER(thread, client, &peer);
    if (peer) {
	if (thread->ipc_state.client_effective_client_sid)
	    peer_esid = thread->ipc_state.client_effective_client_sid;
	else
	    peer_esid = peer->ipc_state.effective_client_sid;

	if (CHECKIPC(peer->sob.sid, peer_esid,
		 thread->sob.sid, thread->ipc_state.effective_server_sid)) {
	    if (!try_remove) {
		thread->ipc_state.effective_server_sid = thread->sob.sid;
		thread->ipc_state.client_effective_client_sid = SECSID_NULL;
		link_reinit(&thread->ipc_state.client_link);
	    } else {
		CHECKRETAINED(peer->sob.sid, peer_esid,
		   thread->sob.sid, thread->ipc_state.effective_server_sid);
	    }
	}
    } else if ((revoke & SUBJECT_SUBJECT__SPECIFY_SERVER) &&
	       ssid != tsid &&
	       thread->sob.sid == ssid &&
	       thread->ipc_state.effective_server_sid == tsid) {
	if (!try_remove)
	    thread->ipc_state.effective_server_sid = thread->sob.sid;
	else
	    retained |= SUBJECT_SUBJECT__SPECIFY_SERVER;
    }
    if (try_remove)
	*out_retained |= retained;

    return;
}


#undef CHECKIPC
#define CHECKIPC(rcsid, ecsid, rssid, essid) { \
       if (rcsid != ecsid) { \
            rc = avc_has_permission_reset(SECCLASS_SUBJECT, \
                                          rcsid,  \
                                          SECCLASS_SUBJECT, \
                                          ecsid,  \
                                          SUBJECT_SUBJECT__SPECIFY_CLIENT, \
                                          &revoke); \
            if (rc) \
                return rc; \
        } \
        if (!revoke) { \
            rc = avc_has_permission_reset(SECCLASS_SUBJECT, \
                                          ecsid,  \
                                          SECCLASS_SUBJECT, \
                                          essid, \
                                          SUBJECT_SUBJECT__CONNECT, \
                                          &revoke); \
            if (rc) \
                return rc; \
        } \
        if (!revoke && rssid != essid) { \
            rc = avc_has_permission_reset(SECCLASS_SUBJECT, \
                                          rssid, \
                                          SECCLASS_SUBJECT, \
                                          essid, \
                                          SUBJECT_SUBJECT__SPECIFY_SERVER, \
                                          &revoke); \
            if (rc) \
                return rc; \
        } \
     }


int
thread_ipc_reset(struct s_thread * thread, oskit_bool_t * out_reset)
{
    oskit_bool_t    revoke = FALSE;
    struct s_thread *peer = NULL;
    security_id_t   peer_esid;
    int             rc = 0;

    *out_reset = FALSE;

    /* thread is locked and stopped on entry */
	
    GETPEER(thread, server, &peer);
    if (peer) {
	if (thread->ipc_state.server_effective_server_sid)
	    peer_esid = thread->ipc_state.server_effective_server_sid;
	else
	    peer_esid = peer->ipc_state.effective_server_sid;

	CHECKIPC(thread->sob.sid, thread->ipc_state.effective_client_sid,
		 peer->sob.sid, peer_esid);

	if (revoke) {
	    thread->ipc_state.effective_client_sid = thread->sob.sid;
	    thread->ipc_state.server_effective_server_sid = SECSID_NULL;
	    link_reinit(&thread->ipc_state.server_link);
	    *out_reset = TRUE;
	}
    } else if (thread->sob.sid != thread->ipc_state.effective_client_sid) {
	rc = avc_has_permission_reset(SECCLASS_SUBJECT,
				      thread->sob.sid,
				      SECCLASS_SUBJECT,
				      thread->ipc_state.effective_client_sid,
				      SUBJECT_SUBJECT__SPECIFY_CLIENT,
				      &revoke);
	if (rc)
	    return rc;

	if (revoke) {
	    thread->ipc_state.effective_client_sid = thread->sob.sid;
	    *out_reset = TRUE;
	}
    }
    GETPEER(thread, client, &peer);
    if (peer) {
	if (thread->ipc_state.client_effective_client_sid)
	    peer_esid = thread->ipc_state.client_effective_client_sid;
	else
	    peer_esid = peer->ipc_state.effective_client_sid;

	CHECKIPC(peer->sob.sid, peer_esid, thread->sob.sid, thread->ipc_state.effective_server_sid);

	if (revoke) {
	    thread->ipc_state.effective_server_sid = thread->sob.sid;
	    thread->ipc_state.client_effective_client_sid = SECSID_NULL;
	    link_reinit(&thread->ipc_state.client_link);
	    *out_reset = TRUE;
	}
    } else if (thread->sob.sid != thread->ipc_state.effective_server_sid) {
	rc = avc_has_permission_reset(SECCLASS_SUBJECT,
				      thread->sob.sid,
				      SECCLASS_SUBJECT,
				      thread->ipc_state.effective_server_sid,
				      SUBJECT_SUBJECT__SPECIFY_SERVER,
				      &revoke);
	if (rc)
	    return rc;

	if (revoke) {
	    thread->ipc_state.effective_server_sid = thread->sob.sid;
	    *out_reset = TRUE;
	}
    }
    return 0;
}
#endif
