/*
 * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * These are the client-side stubs for the kserver-based device drivers.
 */

#include <fluke/sac/fdev.h>		/* cp_fdev_* */
#include <fluke/nest/server/fdev.h>	/* nest_fdev_* */
#include <fluke/ipc.h>
#include <fluke/ref.h>
#include <fluke/thread.h>

#include "debug.h"
#include <malloc.h>

void
cp_fdev_registerhandler(mom_ref_t obj, unsigned short irq, unsigned flags,
		mom_exc_t *env)
{
	fluke_ref_t *client_ref;

	env->_type = exc_mom_no_exception;

	/* create an empty reference */
	client_ref = (fluke_ref_t *)smemalign(FLUKE_REF_SIZE, FLUKE_REF_SIZE);
	assert(client_ref);

	/* create a reference to me, since I will handle the interrupts */
	fluke_ref_create(client_ref);
	fluke_thread_reference(fluke_thread_self(), client_ref);

	/* have that thread get interrupts */
	my_fdev_irq_register(client_ref, irq, flags);

	fluke_ref_destroy(client_ref);
	sfree(client_ref, sizeof(*client_ref));
}

void
cp_fdev_unregisterhandler(mom_ref_t obj, unsigned short irq, mom_exc_t *env)
{
	fluke_ref_t *client_ref;

	env->_type = exc_mom_no_exception;

	client_ref = (fluke_ref_t *)smemalign(FLUKE_REF_SIZE, FLUKE_REF_SIZE);
	assert(client_ref);

	/* create a reference to me */
	fluke_ref_create(client_ref);
	fluke_thread_reference(fluke_thread_self(), client_ref);

	/* unregister the thread */
	my_fdev_irq_unregister(client_ref, irq);

	fluke_ref_destroy(client_ref);
	sfree(client_ref, sizeof(*client_ref));
}


oskit_bool_t
cp_fdev_ioalloc(mom_ref_t obj, fluke_size_t start, fluke_size_t size,
		mom_exc_t *env)
{
	env->_type = exc_mom_no_exception;
	return nest_fdev_ioalloc(obj, start, size, env);
}

void
cp_fdev_iofree(mom_ref_t obj, fluke_size_t start, fluke_size_t size,
		mom_exc_t *env)
{
	env->_type = exc_mom_no_exception;
	nest_fdev_iofree(obj, start, size, env);
}

void
cp_fdev_irq_enable(mom_ref_t obj, oskit_u16_t irq, mom_exc_t *env)
{
	env->_type = exc_mom_no_exception;
	nest_fdev_irq_enable(obj, irq, env);
}

void
cp_fdev_irq_disable(mom_ref_t obj, oskit_u16_t irq, mom_exc_t *env)
{
	env->_type = exc_mom_no_exception;
	nest_fdev_irq_disable(obj, irq, env);
}



/*
 * This overrides the one in fdev that calls cp_parent_getfdev()
 */
void
get_fdev_port(mom_ref_t *device, mom_exc_t *env)
{
	env->_type = exc_mom_no_exception;
	*device = MOM_REF_NULL;
}
