/*
 * Copyright (c) 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Contributed by the Computer Security Research division,
 * INFOSEC Research and Technology Office, NSA.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * kserver access vector cache support.
 */

#if FLASK

#include <fluke/nest/server/server.h>
#include <fluke/nest/server/avc_secure.h>
#include <fluke/nest/server_avc.h>
#include "kserver/kserver.h"

static mom_class_t kserver_avc_class;
static mom_obj_header_t *kserver_avc;

void
kserver_avc_init(oskit_avc_t **avc)
{
    int rc;


    nest_server_avc_init();
    rc = nest_server_avc_com_init(avc);
    assert(!rc);
    
    rc = mom_class_create(nest_module,
			  NULL,
			  nest_Avc_1_0_server_secure,
			  &kserver_avc_class);
    assert(!rc);
    
    rc = mom_obj_create(nest_module, kserver_avc_class,
			0, sizeof(mom_obj_header_t), &kserver_avc);
    assert(!rc);
}


static security_class_pair_t pairs[] = {
{ SECCLASS_SUBJECT, SECCLASS_SUBJECT },
{ SECCLASS_SUBJECT, SECCLASS_MEMORY},
{ SECCLASS_SUBJECT, SECCLASS_FILESYSTEM },
{ SECCLASS_SUBJECT, SECCLASS_FILE }, 
{ SECCLASS_SUBJECT, SECCLASS_DIR },
{ SECCLASS_SUBJECT, SECCLASS_OPENFILE },
{ SECCLASS_SUBJECT, SECCLASS_AVC } 
};
#define NPAIRS ((sizeof pairs)/sizeof(security_class_pair_t))

oskit_bool_t ss_up = FALSE;

mom_ref_t
nest_parent_get_avc(nest_Parent_1_0_t *_obj,
		    fluke_seq_security_class_pair **fpairs,
		    mom_exc_t *_ev)
{
	mom_ref_t ref;
	int rc, i;

	rc = mom_obj_reference(kserver_avc, &ref);
	if (rc) {
		mom_set_exc(_ev, rc);
		return MOM_REF_NULL;
	}

	ss_up = TRUE;

	*fpairs = mom_flick_alloc(sizeof(fluke_seq_security_class_pair));
	(*fpairs)->_length = NPAIRS;
	(*fpairs)->_buffer = mom_flick_alloc(NPAIRS * 
				     sizeof(fluke_security_class_pair));
    
	for (i = 0; i < NPAIRS; i++)
	  {
	    (*fpairs)->_buffer[i].sclass = pairs[i].sclass;
	    (*fpairs)->_buffer[i].tclass = pairs[i].tclass;
	  }

	return ref;
}    


/*
 * Special versions of the nest_server_avc_* functions
 * for the kserver.  These functions replace the 
 * definitions in libnest.
 */

oskit_bool_t nest_server_avc_init(void)
{
    return TRUE;
}


oskit_error_t nest_server_avc_has_permission(security_class_t sclass,
					    security_id_t ssid,
					    security_class_t tclass,
					    security_id_t tsid,
					    access_vector_t requested)
{
    return private_avc_has_permission(sclass, ssid, tclass, tsid, requested);
}


oskit_error_t nest_server_avc_add(security_class_t sclass,
				  security_id_t	ssid,
				  security_class_t tclass,
				  security_id_t tsid,
				  access_vector_t allowed,
				  access_vector_t decided,
				  oskit_u32_t expiration,
				  oskit_bool_t wired,
				  oskit_bool_t replace,
				  oskit_u32_t seqno)
{
    return private_avc_add(sclass, ssid, tclass, tsid, 
			   allowed,decided,expiration,
			   wired,replace, seqno);    
}


oskit_error_t nest_server_avc_remove(security_class_t sclass,
				     security_id_t ssid,
				     security_class_t tclass,
				     security_id_t tsid,
				     oskit_bool_t removewired)
{
    return private_avc_remove(sclass,ssid,tclass,tsid,
			      removewired);    
}


oskit_error_t nest_server_avc_revoke(security_class_t sclass,
				     security_id_t ssid,
				     security_class_t tclass,
				     security_id_t tsid,
				     access_vector_t revoke,
				     oskit_bool_t revokewired,
				     oskit_u32_t seqno)
{
    return private_avc_revoke(sclass,ssid,tclass,tsid,revoke,
			      revokewired, seqno);    
}


oskit_error_t nest_server_avc_try_remove(security_class_t sclass,
                                     security_id_t ssid,
                                     security_class_t tclass,
                                     security_id_t tsid,
                                     access_vector_t permissions,
                                     oskit_bool_t removewired,
                                     oskit_u32_t seqno,
                                     access_vector_t *out_retained)
{
    return private_avc_try_remove(sclass,ssid,tclass,tsid,permissions,
                                  removewired, seqno, out_retained);
}


void
nest_avc_add(nest_Avc_1_0_t *_obj, 
	     fluke_security_class_t sourceclass, 
	     fluke_security_class_t targetclass, 
	     fluke_security_id_t sourcesid,
	     fluke_security_id_t targetsid,
	     fluke_access_vector_t allowed, 
	     fluke_access_vector_t decided, 
	     fluke_time_t expiration, 
	     oskit_bool_t wired, 
	     oskit_bool_t replace, 
	     fluke_u32_t seqno,
	     const security_id_t _csid, 
	     FLASK_SSID_SARG
	     mom_exc_t *_ev)
{
    int rc;

    MOMCHECKOBJRET(_csid,AVC,_obj,ADD,_ev,);

    rc = nest_server_avc_add(sourceclass,
			     sourcesid,
			     targetclass,
			     targetsid,
			     allowed,
			     decided,
			     expiration,
			     wired,
			     replace,
			     seqno);
    mom_set_exc(_ev,rc);
}


void
nest_avc_remove(nest_Avc_1_0_t *_obj, 
		fluke_security_class_t sourceclass, 
		fluke_security_class_t targetclass, 
		fluke_security_id_t sourcesid,
		fluke_security_id_t targetsid,
		oskit_bool_t removewired, 
		const security_id_t _csid, 
		FLASK_SSID_SARG
		mom_exc_t *_ev)
{
    int rc;

    MOMCHECKOBJRET(_csid,AVC,_obj,REMOVE,_ev,);

    rc = nest_server_avc_remove(sourceclass,
				sourcesid,
				targetclass,
				targetsid,
				removewired);
    mom_set_exc(_ev,rc);
}


void
nest_avc_revoke(nest_Avc_1_0_t *_obj, 
		fluke_security_class_t sourceclass, 
		fluke_security_class_t targetclass, 
		fluke_security_id_t sourcesid,
		fluke_security_id_t targetsid,
		fluke_access_vector_t revoke,
		oskit_bool_t revokewired, 
		oskit_u32_t seqno,
		const security_id_t _csid, 
		FLASK_SSID_SARG
		mom_exc_t *_ev)
{
    int rc;

    MOMCHECKOBJRET(_csid,AVC,_obj,REVOKE,_ev,);

    rc = nest_server_avc_revoke(sourceclass,
				sourcesid,
				targetclass,
				targetsid,
				revoke,
				revokewired,
				seqno);
    mom_set_exc(_ev,rc);
}


void
nest_avc_try_remove(nest_Avc_1_0_t *_obj,
                fluke_security_class_t sourceclass,
                fluke_security_class_t targetclass,
                fluke_security_id_t sourcesid,
                fluke_security_id_t targetsid,
                fluke_access_vector_t permissions,
                oskit_bool_t removewired,
                oskit_u32_t seqno,  
                fluke_access_vector_t *out_retained,
                const security_id_t _csid,
                FLASK_SSID_SARG
                mom_exc_t *_ev)  
{   
    int rc;
    
    MOMCHECKOBJRET(_csid,AVC,_obj,TRY_REMOVE,_ev,);
    
    rc = nest_server_avc_try_remove(sourceclass,
                                sourcesid,
                                targetclass,
                                targetsid,
                                permissions, 
                                removewired,
                                seqno,   
                                out_retained);
    mom_set_exc(_ev,rc);
}               


void
nest_avc_reset(nest_Avc_1_0_t *_obj, 
	       oskit_u32_t seqno,
	       const security_id_t _csid, 
	       FLASK_SSID_SARG
	       mom_exc_t *_ev)
{
    int rc;

    MOMCHECKOBJRET(_csid,AVC,_obj,RESET,_ev,);

    rc = private_avc_reset(seqno);
    mom_set_exc(_ev,rc);
}    


typedef struct 
{
    oskit_avc_t	avci;
    unsigned count;
} avc_cache_t;


static avc_cache_t avc_cache;

static struct oskit_avc_ops avc_ops;

oskit_error_t nest_server_avc_com_init(oskit_avc_t **out_avc)
{
    avc_cache.avci.ops = &avc_ops;
    avc_cache.count = 1;
    *out_avc = &avc_cache.avci;
    ++avc_cache.count;
    return 0;
}

/*
 * oskit_avc methods
 */

static OSKIT_COMDECL avc_query(oskit_avc_t *a,
				const struct oskit_guid *iid,
				void **out_ihandle)
{
    avc_cache_t *avc;

    avc = (avc_cache_t*)a;
    assert(avc);
    assert(avc->count);
    
    if (memcmp(iid, &oskit_iunknown_iid, sizeof(*iid)) == 0 ||
	memcmp(iid, &oskit_avc_iid, sizeof(*iid)) == 0) { 
	*out_ihandle = &avc->avci;
	++avc->count;
	return 0;
    }

    *out_ihandle = NULL;
    return OSKIT_E_NOINTERFACE;    
}


static OSKIT_COMDECL_U avc_addref(oskit_avc_t *a)
{
    avc_cache_t *avc;

    avc = (avc_cache_t*)a;
    assert(avc);
    assert(avc->count);

    return ++avc->count;
}


static OSKIT_COMDECL_U avc_release(oskit_avc_t *a)
{
    avc_cache_t *avc;
    unsigned newcount;
    
    avc = (avc_cache_t*)a;
    assert(avc);
    assert(avc->count);

    newcount = --avc->count;
    if (newcount == 0)
    {
	/* for now, do nothing */
    }

    return newcount;
}


static OSKIT_COMDECL avc_has_permission(oskit_avc_t *a,
					security_class_t sclass,
					security_id_t ssid,
					security_class_t tclass,
					security_id_t tsid,
					access_vector_t requested)
{
    avc_cache_t *avc;
    
    avc = (avc_cache_t*)a;
    assert(avc);
    assert(avc->count);

    return nest_server_avc_has_permission(sclass,ssid,tclass,tsid,requested);
}


static OSKIT_COMDECL avc_compute_access_vector(oskit_avc_t *a,
					security_class_t sclass,
					security_id_t ssid,
					security_class_t tclass,
					security_id_t tsid,
					access_vector_t requested,
					access_vector_t *out_allowed)
{
    avc_cache_t *avc;
    
    avc = (avc_cache_t*)a;
    assert(avc);
    assert(avc->count);

    return OSKIT_E_NOTIMPL; 
}


static OSKIT_COMDECL avc_add_callback(oskit_avc_t *a,
				      struct oskit_avcomcallback *c,
				      oskit_u32_t etype,
				      security_class_t sclass,
				      security_id_t ssid,
				      security_class_t tclass,
				      security_id_t tsid,
				      access_vector_t permissions)
{
    avc_cache_t *avc;
    
    avc = (avc_cache_t*)a;
    assert(avc);
    assert(avc->count);

    return OSKIT_E_NOTIMPL;
}


static OSKIT_COMDECL avc_remove_callback(oskit_avc_t *a,
					 struct oskit_avcomcallback *c)

{
    avc_cache_t *avc;
    
    avc = (avc_cache_t*)a;
    assert(avc);
    assert(avc->count);

    return OSKIT_E_NOTIMPL;
}

static struct oskit_avc_ops avc_ops = {
    avc_query,
    avc_addref,
    avc_release,
    avc_has_permission,
    avc_compute_access_vector,
    avc_add_callback,
    avc_remove_callback
};

#endif
