/*
 * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * kserver support for simple console device.
 */

#include "kserver/kserver.h"

#include "config_fdev.h"
#ifdef CONFIG_FDEV
#include "config_fdev_console.h"
#endif
#ifdef DEMO_GUI
#include <fluke/nest/server/demogui.h>
#endif

/*
 * Hand out `console_tty_ref' to children as stdin, stdout, and stderr.
 */
fluke_seq_InitFD *
nest_parent_get_init_fds(nest_Parent_1_0_t *_obj,
			 mom_exc_t *env)
{
	fluke_seq_InitFD *ret;

	DPRINTF(" %s\n",__FUNCTION__);

	/* allocate space for 3 InitFD objects */
	ret = mom_flick_alloc(sizeof(*ret));
	if (!ret)
		goto nomem;
	ret->_buffer = mom_flick_alloc(3 * sizeof(fluke_InitFD));
	if (!ret->_buffer) {
		mom_flick_free(ret);
	nomem:
		mom_set_exc(env, ENOMEM);
		return NULL;
	}
	ret->_length = 3;

	ret->_buffer[0].fd = STDIN_FILENO;
	ret->_buffer[1].fd = STDOUT_FILENO;
	ret->_buffer[2].fd = STDERR_FILENO;
	ret->_buffer[0].stream = console_tty_ref;
	ret->_buffer[1].stream = console_tty_ref;
	ret->_buffer[2].stream = console_tty_ref;
	mom_ref_acquire(console_tty_ref);
	mom_ref_acquire(console_tty_ref);
	mom_ref_acquire(console_tty_ref);

	return ret;
}


/*
 * The following code is only used if fdev support is disabled in the kernel;
 * if fdev support is available, ks_fdev.c defines the console tty device.
 */

#ifndef CONFIG_FDEV_CONSOLE

#include <fluke/nest/server/io.h>
#include <sys/stat.h>
#include <string.h>

mom_ref_t console_tty_ref;
mom_class_t kserver_console_class;

void
kserver_console_init()
{
	mom_obj_header_t *consobj;
	int rc;

#ifdef DEMO_GUI
	demogui_init(nest_module);
	consobj = demogui_allocate();
	nest_demogui_cmd(consobj, "setdelay 10", 0);
#if FLASK
	nest_demogui_cmd(consobj, "settitle ss", 0);
	nest_demogui_cmd(consobj, "setnodename ss", 0);
#else
	nest_demogui_cmd(consobj, "settitle fluke-init", 0);
	nest_demogui_cmd(consobj, "setnodename fluke-init", 0);
#endif
#else
	rc = mom_class_create(nest_module, NULL, /* no cleanup */
			      nest_StreamIO_server, &kserver_console_class);
	assert(!rc);

	rc = mom_obj_create(nest_module, kserver_console_class,
			    0, sizeof(*consobj), &consobj);
	assert(!rc);
#endif

	rc = mom_obj_reference(consobj, &console_tty_ref);
	assert(!rc);

	mom_obj_release_ref(consobj);
}

#ifndef DEMO_GUI

/*
 * fluke::StreamIO server functions for the trivial console.
 * There is just one MOM object representing the console; it has no state.
 */

fluke_seq_octet *
nest_io_read(nest_StreamIO_t *file, size_t length, mom_exc_t *env)
{
	fluke_seq_octet *ret;
	char *p;
	int i;
	int c;

	if ((ret = mom_flick_alloc(sizeof(*ret)))) {
		if ((ret->_buffer = mom_flick_alloc(length))) {
			/* was able to allocate memory for the buffers */

			for (i = 0, p = ret->_buffer; i < length; i++, p++) {
				c = getchar();
				if (c < 0)
					break;
				*p = (char)c;
			}

			ret->_length = i;

		} else {
			mom_set_exc(env, ENOMEM);
			mom_flick_free(ret);
			ret = NULL;
		}
	} else {
		mom_set_exc(env, ENOMEM);
	}
	return ret;

}

size_t
nest_io_write(nest_StreamIO_t *file,
	      const fluke_seq_octet *data, mom_exc_t *env)
{
	printf("%.*s", (int) data->_length, data->_buffer);
	return data->_length;
}

size_t
nest_io_readable(nest_StreamIO_t *file, mom_exc_t *env)
{
	printf("%s\n",__FUNCTION__);  mom_set_exc(env, ENOSYS);
	return -1;
}

void
nest_io_poll(nest_StreamIO_t *file, oskit_u32_t *mask,
		mom_exc_t *env)
{
	printf("%s\n",__FUNCTION__);  mom_set_exc(env, ENOSYS);
}

void
nest_io_select(nest_StreamIO_t *file, oskit_u32_t *mask,
		const mom_ref_t callback, mom_ref_t *lease, mom_exc_t *env)
{
	printf("%s\n",__FUNCTION__);  mom_set_exc(env, ENOSYS);
}

void
nest_io_getattr(nest_PosixIO_t *file,
		struct fluke_attr_t *attributes, mom_exc_t *env)
{
	memset(attributes, 0, sizeof *attributes);
	attributes->mode = S_IFCHR | 0777;
}

#if FLASK
void
nest_io_getattr_secure(nest_PosixIO_t *file,
		       struct fluke_attr_t *attributes,
		       fluke_security_class_t *sclass,
		       fluke_security_id_t *sid,		       
		       mom_exc_t *env)
{
	memset(attributes, 0, sizeof *attributes);
	attributes->mode = S_IFCHR | 0777;
	*sclass = SECCLASS_FILE;
	mom_obj_sid((void*)file, (security_id_t*)sid);
}
#endif 

oskit_s32_t
nest_io_pathconf(nest_PosixIO_t *file, oskit_s32_t name,
		 mom_exc_t *env)
{
	printf("%s\n",__FUNCTION__);  mom_set_exc(env, ENOSYS);
	return -1;
}

fluke_off_t
nest_io_seek(nest_SeekableIO_t *_obj,
	     fluke_off_t offset, fluke_seek_t whence,
	     mom_exc_t *_ev)
{
	mom_set_exc(_ev, ESPIPE);
	return -1L;
}

/* required by libnest io-server.o, never called */
void nest_io_select_callback(nest_SelectListener_t *_obj,
			     oskit_u32_t mask, mom_exc_t *_ev)
{
	otsan();
}

#endif	/* !DEMO_GUI */

#endif	/* !CONFIG_FDEV_CONSOLE */
