/*
 * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * fdev device support for kserver
 */

#include "config_fdev.h"

#ifdef CONFIG_FDEV

#include "fdev/eth.h"

#include "kserver/kserver.h"

#include <fluke/nest/server/com.h>

#include <oskit/dev/dev.h>
#include <oskit/dev/blk.h>
#include <oskit/dev/ethernet.h>
#include <oskit/dev/bus.h>

#include <oskit/dev/linux.h>
#include <oskit/dev/freebsd.h>

#include <oskit/fs/file.h>	/* XXX for FLUX_O_* */
#include <oskit/fs/dir.h>
#include <oskit/fs/soa.h>
#include <fluke/sac/fs.h>
#include <string.h>

#include "config_fdev_console.h"

#ifdef CONFIG_FDEV_CONSOLE
#include <oskit/dev/tty.h>
#include <oskit/io/ttystream.h>

static fluke_mutex_t console_mutex;
static struct nest_com_lock_data console_lock = { NULL, NULL, &console_mutex };
#endif

void fdev_process_lock();
void fdev_process_unlock();


mom_ref_t console_tty_ref;

void attach_fdev_devices();

void
kserver_oskit_dev_init()
{
#ifdef CONFIG_FDEV_CONSOLE
	mom_exc_t exc;
	int ndev;
	oskit_error_t rc;
	oskit_ttydev_t **ttydev;
	oskit_ttystream_t *ttystream;
#endif
	printf("Initializing device drivers...\n");

	DMARK;
	oskit_dev_init();
	DMARK;

	fdev_process_lock();
#if 0
	oskit_linux_dev_init_all();
	oskit_freebsd_init_all();
#else
	oskit_linux_init_scsi_ncr53c8xx();
	oskit_linux_init_scsi_aic7xxx();
	oskit_linux_init_ide();
	oskit_linux_init_ethernet_tulip();
	oskit_linux_init_ethernet_eepro100();
	DMARK;
#ifdef CONFIG_FDEV_CONSOLE
	oskit_freebsd_init_sio();	/* XXX use _sc for vga console */
#endif
#endif
#if 0
	DMARK;
	oskit_dump_drivers();
	DMARK;
#endif

	printf("Probing devices...\n");
	oskit_dev_probe();

	fdev_io_free_unused();

	DMARK;
	oskit_dump_devices();
	fdev_process_unlock();

#ifdef CONFIG_FDEV_CONSOLE
	/*
	 * Find all TTY device nodes.
	 */
	ndev = osenv_device_lookup(&oskit_ttydev_iid, (void***)&ttydev);
	if (ndev <= 0)
		panic("no TTY devices found!");

	/*
	 * Open the first TTY device found, which should be the display.
	 */
	rc = oskit_ttydev_open(ttydev[0], OSKIT_O_RDWR, &ttystream);
	if (rc)
		panic("unable to open tty0: %x", rc);
	assert(ttystream);

	while (--ndev > 0)
		oskit_device_release(ttydev[ndev]);
	osenv_mem_free(ttydev, OSENV_AUTO_SIZE, 0);

	/*
	 * Wrap the ttystream COM object as a MOM object and
	 * prepare a ref to it that we will give our children.
	 */
	rc = nest_com_termios_init();
	assert(!rc);
	fluke_mutex_create(&console_mutex);
	fluke_debug_label_flob(&console_mutex, "console_mutex");
	exc._type = exc_mom_no_exception;
	console_tty_ref = nest_com_instantiate_ref(nest_com_ttystream_class,
						    ttystream,
						    &console_lock,
						    &exc);
	assert(exc._type == exc_mom_no_exception);
#endif

	/*
	 * Populate /dev with device nodes.
	 */
	attach_fdev_devices();

	kserver_clock_init();
}
#endif	/* CONFIG_FDEV */
