/*
 * Copyright (c) 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * These routines provide memory allocation for the MOM infrastructure.
 * Note smemalign & sfree are do_in_supervisor_mode'd in ../malloc.c
 */

#include "kserver/kserver.h"


void *
nest_alloc(size_t alignment, size_t size)
{
	/*
	 * NOTE! nest_alloc takes log2(alignment), but
	 * smemlaign takes alignment in bytes directly!
	 */
	return smemalign(1 << alignment, size);
}


void
nest_free(void *mem, size_t size)
{
	return sfree(mem, size);
}


/* XXX could be made smarter, so it doesn't always copy to a new block.  */
void *
nest_realloc(void *buf, size_t align, size_t old_size, size_t new_size)
{
	void *new;

	new = nest_alloc(align, new_size);
	if (new && old_size) {
		memcpy(new, buf, old_size);
		nest_free(buf, old_size);
	}

	return new;
}

