/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_OBINFO_H_
#define _FLUKE_KERNEL_OBINFO_H_

struct s_ob;

struct obinfo {
	oskit_size_t	u_size;
	oskit_size_t	s_size;
	oskit_size_t	s_align;
	void		(*u_sanify)(struct s_ob *);
};

extern const struct obinfo obinfo[];

#define OB_USE_INLINE

#ifdef OB_USE_INLINE
#define OB_INFO_FIELDS	\
	oskit_size_t	u_size; \
	oskit_size_t	s_size;

#define OB_INFO_ENTER(ob)	\
	(ob)->u_size = obinfo[(ob)->type].u_size; \
	(ob)->s_size = obinfo[(ob)->type].s_size;

#define OB_USIZE(ob)	((ob)->u_size)
#define OB_SSIZE(ob)	((ob)->s_size)
#endif

#ifdef OB_USE_POINTER
#define OB_INFO_FIELDS	\
	const struct obinfo	*obptr;

#define OB_INFO_ENTER(ob)	\
	(ob)->obptr = &obinfo[(ob)->type];

#define OB_USIZE(ob)	((ob)->obptr->u_size)
#define OB_SSIZE(ob)	((ob)->obptr->s_size)
#endif

#ifdef OB_USE_ARRAY
#define OB_INFO_FIELDS
#define OB_INFO_ENTER(ob)
#define OB_USIZE(ob)	(obinfo[(ob)->type].u_size)
#define OB_SSIZE(ob)	(obinfo[(ob)->type].s_size)
#endif

/* using a type instead of an object */
#define OBT_USIZE(type)	(obinfo[type].u_size)
#define OBT_SSIZE(type)	(obinfo[type].s_size)
#define OBT_SALIGN(type) (obinfo[type].s_align)

#endif /* _FLUKE_KERNEL_OBINFO_H_ */
