/*
 * Copyright (c) 1995-1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLOSS_KERNEL_FLUKE_PMAP_H_
#define _FLOSS_KERNEL_FLUKE_PMAP_H_

#include <oskit/machine/types.h>
#include <fluke/vm.h>

/* Machine-dependent code must define struct pmap and struct pmap_frag.  */

struct s_thread;
struct s_task;

/*
 * Initialize the per-task PMAP layer state in a new task being created.
 * The task must be locked throughout.
 */
int task_pmap_state_init(struct s_task *task);

/*
 * Destroy the per-task PMAP layer state in a task being destroyed.
 * The task must be locked throughout.
 */
void task_pmap_state_destroy(struct s_task *task);

void pmap_enter(struct s_task *task, oskit_addr_t va, oskit_addr_t pa,
		oskit_size_t size, fluke_prot_t prot);
void pmap_remove(struct s_task *task, oskit_addr_t va, oskit_size_t size);
void pmap_protect(struct s_task *task, oskit_addr_t va, oskit_size_t size,
		  fluke_prot_t old_prot, fluke_prot_t new_prot);

/*
 * Make the current thread run using the specified task's page tables.
 * Assumes the thread doesn't already have a user-level address space.
 * Thread and task are locked throughout.
 * Called by s_task.c when a thread's task pointer is set.
 */
void pmap_activate(struct s_thread *cur_thread, struct s_task *task);

/*
 * Make the current thread run without any user-mode address space.
 * Thread and task are locked throughout.
 * Called by s_task.c when a thread's task pointer is cleared.
 */
void pmap_deactivate(struct s_thread *cur_thread, struct s_task *task);

/*
 * Initialize the page mapping layer at boot time.
 */
void pmap_boot(void);

#endif /* _FLOSS_KERNEL_FLUKE_PMAP_H_ */
