/*
 * Copyright (c) 1996-1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_PROG_H_
#define _FLUKE_KERNEL_PROG_H_

#define PROG_NAME	"/sbin/fluke-init"

/*
 * Called in supervisor mode after all other kernel initialization,
 * just before starting to run "real" threads.
 * Creates the first actual user-mode thread,
 * which starts executing the following function.
 */
void prog_boot(char **argv, char **envp);

/*
 * Prompt the console for command line arguments for the initial task and
 * return an arg vector pointing into static storage.
 */
void prog_boot_parse_cmdline(char ***argvpp, char ***envpp);

/*
 * First function to be called in user mode,
 * in the "root task" that runs in the kernel's address space.
 * Initializes the kserver and then loads and starts the first
 * independent user program.
 */
void prog_start(void);

/*
 * XXX shouldn't be here.
 */
void do_in_supervisor_mode(int (*func)(void *arg), void *arg);
#include <fluke/sac/pthread.h>	/* XXX */
int supervisor_pthread_create(pthread_t *out_thread,
			      const pthread_attr_t *attr,
			      void *(*start_routine)(void *),
			      void *start_arg);


#endif /* _FLUKE_KERNEL_PROG_H_ */
