/*
 * Copyright (c) 1996-1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include <pthread.h>

#include "prog.h"
#include "s_thread.h"
#include "debug.h"

#include "config_smp.h"
#include "config_user_sched.h"

char **global_argv;	/* XXX hack */
extern char **environ;

/*
 * This function is called in the root thread of the boot CPU.
 * The thread is fully set up except for the user-mode register
 * state to return to.
 * This is the last function called from main().
 */
void prog_boot(char **argv, char **envp)
{
	fluke_thread_state_t ts;

	/*
	 * When we return to user mode, we will call prog_start.
	 * It will call kserver with args from the boot loader.
	 */
	EXC_SET_IP(&startup_thread, (oskit_addr_t) &prog_start);

	/*
	 * Do a proper set-state operation with the initial state.
	 * This will set any magic bits needed for kserver threads.
	 */
	s_thread_lock(&startup_thread);
	s_thread_get_state(&startup_thread, &ts);

	/*
	 * XXX this could be changed to push argv/envp on the stack
	 * via some machine-dependent call, but I don't think it is
	 * worth the trouble.
	 */
	global_argv = argv;
	environ = envp;

	s_thread_set_state(&startup_thread, &ts);
	s_thread_unlock(&startup_thread);

	/* Start the dispatcher running this first thread */
	disp_start(&startup_thread);

	/* That never returns */
	otsan();
}
