/*
 * Copyright (c) 1995, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLOSS_KERNEL_FLUKE_SUPER_COND_H_
#define _FLOSS_KERNEL_FLUKE_SUPER_COND_H_

#include "s_ob.h"
#include "cond.h"

typedef struct s_cond
{
	struct s_ob		sob;
	oskit_u32_t		flags;
	link_t			inheritor_link;
	cond_t			cond;
} s_cond_t;

#define s_cond_lock(c)		s_ob_lock(&(c)->sob)
#define s_cond_unlock(c)	s_ob_unlock(&(c)->sob)
#define assert_s_cond_locked(c) assert_s_ob_locked(&(c)->sob)

struct s_mutex;
struct fluke_cond_state;

int s_cond_create(
	oskit_addr_t		uva,
	fluke_hash_t		hash,
	s_cond_t		**out_cond);

int s_cond_destroy(
	s_cond_t		*cond);

int s_cond_get_state(
	s_cond_t		*cond,
	struct fluke_cond_state	*state,
	struct link		*inheritor_link);

int s_cond_set_state(
	s_cond_t		*cond,
	struct fluke_cond_state	*state,
	struct link		*inheritor_link);

/*
 * Wait on the specified user-exported condition variable.
 * On entry, the cond is locked.
 * On return the cond is unlocked.
 * This routine does _not_ reacquire the mutex after the wait;
 * the caller must do that.
 */
int s_cond_wait(
	s_cond_t		*cond);

/*
 * Signal the specified user-exported condition variable.
 * Condition variable locked on entry and exit.
 */
int s_cond_signal(
	s_cond_t		*cond);

/*
 * Wake up all threads on the specified user-exported condition variable.
 * Condition variable locked on entry and exit.
 */
int s_cond_broadcast(
	s_cond_t		*cond);

#ifdef DEBUG
void s_cond_dump(s_cond_t *cond, int indent);
#endif

#endif /* _FLOSS_KERNEL_FLUKE_SUPER_COND_H_ */
