/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLOSS_KERNEL_FLUKE_S_MUTEX_H_
#define _FLOSS_KERNEL_FLUKE_S_MUTEX_H_

#include "s_ob.h"
#include "cond.h"

#include "config_opt_synch.h"

typedef struct s_mutex
{
	struct s_ob		sob;

	cond_t			cond;
	oskit_u32_t		flags;

#ifndef CONFIG_OPT_SYNCH
	struct link		owner;
	oskit_bool_t		locked;
#define MUTEX_ISLOCKED(m)		((m)->locked)
#define MUTEX_SETLOCKED(m, flag)	((m)->locked = (flag))
#else
	/*
	 * Pointer through which we can access the user part of the mutex.
	 */
	u_mutex_t		*u_mutex;
#endif
} s_mutex_t;

#define s_mutex_lock(m)		s_ob_lock(&(m)->sob)
#define s_mutex_unlock(m)	s_ob_unlock(&(m)->sob)
#define assert_s_mutex_locked(m) assert_s_ob_locked(&(m)->sob)

struct fluke_mutex_state;
struct link;

int s_mutex_create(
	oskit_addr_t		uva,
	fluke_hash_t		hash,
	s_mutex_t		**out_mutex);

int s_mutex_destroy(
	s_mutex_t		*mutex);

int s_mutex_get_state(
	s_mutex_t			*mutex,
	struct fluke_mutex_state	*state,
	struct link			*owner_link);

int s_mutex_set_state(
	s_mutex_t			*mutex,
	struct fluke_mutex_state	*state,
	struct link			*owner_link);

int s_mutex_lockop(
	s_mutex_t		*mutex,
	struct s_thread		*cur_th);

int s_mutex_trylockop(
	s_mutex_t		*mutex);

int s_mutex_unlockop(
	s_mutex_t		*mutex,
	struct s_thread		*cur_th);

#ifdef DEBUG
void s_mutex_dump(s_mutex_t *mutex, int indent);
#endif

#endif /* _FLOSS_KERNEL_FLUKE_S_MUTEX_H_ */
