/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include "config_user_gprof.h"

#ifdef CONFIG_USER_GPROF

#include "s_profil.h"
#include "debug.h"

#include "kalloc.h"
#include "xfer.h"
#include "s_task.h"
#include "s_thread.h"
#include <sys/gmon.h>

/*
 * gprof support code
 */
int
s_profil(oskit_addr_t samples, int size, int offset, int scale)
{
	int rc;

	/* get the current task */
	struct s_task *p;
	struct s_thread *cur_th = current_thread();
	p = cur_th->task_state.task;

	if (scale > (1 << 16)) {
		printf("bad scale in s_profil\n");
		return 0; /* EINVAL */
	}

	if (scale == 0) {
		if (p->p_prof == NULL) {
			printf("profiling already off\n");
			return 0;
		}
		/* turn it off */
		p->p_flag &= ~P_PROFIL;

		/* XXX: others may be still profiling */
		pic_disable_irq(8);
		printf("kernel: turning off profil\n");

		assert(p->p_prof->buffer);
		/*
		 * Copy the kernel's buffer out to the program.
		 * Use the buffer given when profiling was turned on.
		 */
		rc = copyout_local(p->p_prof->buffer,
			p->p_prof->pr_base,
			p->p_prof->pr_size / sizeof(oskit_reg_t));
		if (rc) return rc;

		kfree(p->p_prof->buffer, p->p_prof->pr_size);
		kfree(p->p_prof, sizeof(struct uprof));
		p->p_prof = NULL;
		return 0;
	}

	if (p->p_prof) {
		printf("profiling already on...\n");
		return 0;
	}

	printf("kernel: turning on profil\n");

	/*
	 * Set up for profiling.
	 */
	p->p_prof = kalloc(sizeof(struct uprof));
	if (p->p_prof == NULL)
		return KR_NO_MEMORY;

	p->p_prof->buffer = kalloc(size);
	if (p->p_prof == NULL) {
		kfree(p->p_prof, sizeof(struct uprof));
		return KR_NO_MEMORY;
	}
	p->p_prof->pr_base = samples;
	p->p_prof->pr_size = size;
	p->p_prof->pr_off = offset;
	p->p_prof->pr_scale = scale;

	/*
	 * Copy the program's buffer into the kernel (wired memory)
	 * I need to copy it in in case profiling was previously on,
	 * using the same buffer.
	 */
	rc = copyin_local(p->p_prof->pr_base, p->p_prof->buffer,
		p->p_prof->pr_size / sizeof(oskit_reg_t));
	/* XXX: if rc!=0... */
	if (rc)
		return rc;

	p->p_flag |= P_PROFIL;

	prof_clock_init(OSKIT_PROFHZ);
	return (0);
}

#endif CONFIG_USER_GPROF
