/*
 * Copyright (c) 1995, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLOSS_KERNEL_FLUKE_S_REF_H_
#define _FLOSS_KERNEL_FLUKE_S_REF_H_

#include "s_ob.h"

#include "config_opt_ref.h"

#if FLASK
struct avc_entry;
#endif

typedef struct s_ref
{
	struct s_ob	sob;

	/* Link to the target object.  */
	struct link	link;

#if FLASK
	struct avc_entry *ae;
	unsigned	aever;
#endif

#ifdef CONFIG_OPT_REF
	struct u_ref	*u_ref;	/* Pointer to user-space object.  */
#endif
} s_ref_t;

#define s_ref_lock(r)		s_ob_lock(&(r)->sob)
#define s_ref_unlock(r)		s_ob_unlock(&(r)->sob)

struct fluke_ref_state;

/*
 * Create a NULL reference.
 * Returns with the reference locked.
 */
int s_ref_create(oskit_addr_t uva);

/*
 * Destroy a reference.
 * Reference is locked on entry, unlocked on exit.
 */
void s_ref_destroy(s_ref_t *ref);

/*
 * Check a reference for being null.
 * Sets *rc to 0 if it's null, 1 if not.
 */
int s_ref_check(s_ref_t *ref, int *rc);

struct s_ob *s_ref_follow(s_ref_t *ref);
void s_ref_create_link(s_ref_t *ref, link_t *link);

/*
 * Associate a reference object with a target object.
 * Any old association is first severed.
 * Both the reference and the target object must be locked.
 */
void s_ref_associate(s_ref_t *ref, s_ob_t *target);

/*
 * Associate a reference object with the target object of a link.
 */
void s_ref_link_transfer(link_t *srclink, s_ref_t *ref);

#ifdef DEBUG
void s_ref_dump(s_ref_t *ref, int indent);
#endif

#include "config_inlining.h"
#ifdef CONFIG_INLINING
#define s_ref_follow(ref)		link_follow(&(ref)->link)
#define s_ref_create_link(ref, linkp)	link_create_copy(&(ref)->link, (linkp))

extern inline void s_ref_link_transfer(link_t *srclink, s_ref_t *ref) {
	s_ob_t *target = link_follow(srclink);
	s_ref_associate(ref, target);
	if (target)
		s_ob_unlock(target);
}
#endif

#endif /* _FLOSS_KERNEL_FLUKE_S_REF_H_ */
