/*
 * Copyright (c) 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_SCHED_H
#define _FLUKE_KERNEL_SCHED_H

#include "spin_lock.h"

/* Define sched_state_t */
#include "config_stride_sched.h"
#ifdef CONFIG_STRIDE_SCHED
#include "stride.h"
#else
#include "round_robin.h"
#endif

#include "s_thread.h"
#include <fluke/thread.h>

#include "config_stride_sched.h"

#ifdef CONFIG_STRIDE_SCHED
#include "stride.h"
#else
#include "round_robin.h"
#endif


/*
 * Functions to maintain per-thread scheduler state.
 * These are called only on captured threads.
 */
void thread_sched_set_state(struct s_thread *target,
			    fluke_thread_state_t *new_state);
void thread_sched_get_state(struct s_thread *target,
			    fluke_thread_state_t *state);
void thread_sched_state_init(struct s_thread *th);
void thread_sched_state_destroy(struct s_thread *th);
void thread_sched_canceled(struct s_thread *th);


/*
 * This lock must be held while calling any sched_*
 * function (except sched_boot, which initializes the lock).
 */
extern spin_lock_t sched_lock;

/*
 * Initialize the scheduler data structures.
 */
void sched_boot(void);

/* Glue functions for the dispatcher */

struct s_thread *sched_thread_next(void);
int sched_thread_yield(void);
int sched_ready_queue_empty(void);
void sched_first_thread(struct s_thread *t);
int sched_thread_ready(struct s_thread *s);

#endif
