/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_SPIN_LOCK_H_
#define _FLUKE_KERNEL_SPIN_LOCK_H_

#include "config_smp.h"

#ifdef CONFIG_SMP

/* Include the architecture-specific spin lock implementation.  */
#include <oskit/machine/spin_lock.h>

#define assert_spin_lock_locked(l)	assert(spin_lock_locked(l))
#define assert_spin_lock_unlocked(l)	assert(spin_lock_unlocked(l))

#else /* no SMP support */

#ifdef DEBUG
/* If DEBUG, use fake spin locks, for testing */

typedef unsigned spin_lock_t;

#define SPIN_LOCK_FREE		0x43211234
#define SPIN_LOCK_HELD		0x87655678

#define SPIN_LOCK_INITIALIZER		{ SPIN_LOCK_FREE }
#define assert_spin_lock_locked(l)	assert(*(l) == SPIN_LOCK_HELD)
#define assert_spin_lock_unlocked(l)	assert(*(l) == SPIN_LOCK_FREE)

#define spin_lock_init(l) \
	{ *(l) = SPIN_LOCK_FREE; }
#define spin_lock(l) \
	{ assert_spin_lock_unlocked(l); *(l) = SPIN_LOCK_HELD; }
#define spin_unlock(l) \
	{ assert_spin_lock_locked(l); *(l) = SPIN_LOCK_FREE; }

#else

/* Just compile out all spin locks.  */
typedef struct { } spin_lock_t;
#define SPIN_LOCK_INITIALIZER		{ }
#define spin_lock_init(l)		((void)0)
#define spin_lock(l)			((void)0)
#define spin_unlock(l)			((void)0)
#define assert_spin_lock_locked(l)	((void)0)
#define assert_spin_lock_unlocked(l)	((void)0)

#endif /* DEBUG */
#endif /* CONFIG_SMP */

#endif /* _FLUKE_KERNEL_SPIN_LOCK_H_ */
