/*
 * Copyright (c) 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_STRIDE_H_
#define _FLUKE_KERNEL_STRIDE_H_

#include "config_hier_sched.h"
#include "config_hier_sched_set_parent.h"

#ifdef CONFIG_HIER_SCHED
#include "currency.h"
#endif

#include "prio_queue.h"
#include "link.h"

struct sched_state {
	/* scheduling queue link for thread */
	struct prio_queue_node link;

	/* Tickets asigned to client */
	unsigned tickets;
  
	/* Time it has to wait before receiving the CPU for a
	   quantum. It is inversely proportional to the amount 
	   of tickets held */
	unsigned stride;

	/* Indicates when it should be scheduled. After the
	   client is assigned a quantum this value is increased
	   by stride */
	unsigned pass;
  
	/* Time before or after being assigned a quantum.
	   Adjusted when the client leaves the ready queue */
	int remain;

	/* Variable used to calculate the elapsed execution time */
	unsigned start_time;

#ifdef _STRIDE_PROFILE
	unsigned total_ticks;
	unsigned start_exec_time;
#endif

#ifdef CONFIG_HIER_SCHED 
	currency_state_t curr_state;
#ifdef CONFIG_HIER_SCHED_SET_PARENT

	/* Reference to the parent saved here during syscall */
	link_t sched_link;
#endif
	/* If true this is not a sys call but a kserver call */
	int kserver;
#endif

};
typedef struct sched_state sched_state_t;

#define SCHED_LINK_TO_THREAD(l) \
	((struct s_thread *)((oskit_addr_t)(l) - \
		(oskit_addr_t)&((struct s_thread *)0)->sched_state.link))

/* Operation used by currency.c */
void stride_client_modify(struct s_thread *t, int tickets);

#if defined(CONFIG_HIER_SCHED) && defined(DEBUG)
void stride_dump_tree(void);
int stride_check_tree(void);
#endif

#endif
