/*
 * Copyright (c) 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Special instrumentation entrypoints for our kernel implementation.
 */

#include "config_instrument.h"

#ifndef CONFIG_INSTRUMENT

int sys_instrument_thread()         { return 0; }

#else /* CONFIG_INSTRUMENT is defined */

#include <fluke/machine/syscall.h>
#include <fluke/instrument.h>

#include "kalloc.h"
#include "s_thread.h"
#include "s_instrument.h"

int
sys_instrument_thread() {
	s_thread_t *cur_th = current_thread();
	s_thread_t *dest_th;
	int rc;
	int op;

	op = EXC_SYSPARAM(cur_th, instrument_thread, op);

	rc = task_ob_lookup_type(cur_th,
				 EXC_SYSPARAM(cur_th, instrument_thread,
					      thread),
				 FLUKE_PROT_WRITE, FLUKE_THREAD,
				 (s_ob_t**)&dest_th);
	if (rc)
		return rc;

	switch(op)
	{
	case INSTRUMENT_END:
		s_instrument_thread_end(dest_th);
		break;
	case INSTRUMENT_BEGIN:
		s_instrument_thread_begin(dest_th);
		break;
	case INSTRUMENT_DUMP:
		s_instrument_thread_dump_buf(dest_th);
		break;
	case INSTRUMENT_EVENT1:
		INSTRUMENT_EVENT(dest_th,  INSTRUMENT_EVENT_USER_1, 0)
		break;
	case INSTRUMENT_EVENT2:
		INSTRUMENT_EVENT(dest_th,  INSTRUMENT_EVENT_USER_2, 0)
		break;
	case INSTRUMENT_EVENT3:
		INSTRUMENT_EVENT(dest_th,  INSTRUMENT_EVENT_USER_3, 0)
		break;
	default: /* Unknown Op.  too f-ing bad. */
		break;
	}

	s_thread_unlock(dest_th);
	return 0;
}


#endif /* def'd CONFIG_INSTRUMENT */
	
	
	
	
