/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_X86_ACOUNT_H_
#define _FLUKE_KERNEL_X86_ACOUNT_H_

#include "smp.h"
#include "config_opt_acount.h"

#ifdef CONFIG_OPT_ACOUNT

/*
 * flag that we're overriding the MI portion of fluke's 
 * portable version of atomic counters with our optimized
 * version
 */
#define MD_ACOUNT

/* The Pentium processor has the LOCK XADD combo,
 * so the only thing we need is our counter.  
 * LOCK XADD allows us to add this atomically
 */
typedef struct acount
{
	int		counter;
} acount_t;

/*
 * Initialize the counter
 */
#define _acount_init(ac, value)   (ac)->counter = (value)

/*
 * Increment counter by one.
 * Returns the old value.
 *
 * XXX the lock xaddl instruction will only work on 486 or higher 
 *     processors....
 */
#define _acount_incr(ac)                                         \
	  ({int rc;                                             \
            asm volatile(LOCKPREFIXSTR" xaddl %%eax,(%2)"	\
            	: "=a" (rc) : "a" (1), "r" (&(ac)->counter) : "eax"); \
	    assert(rc >= 0); \
            rc; })

/*
 * Decrement counter by one.
 * Returns the new value.
 * 
 * XXX the lock xaddl instruction will only work on 486 or higher 
 *     processors....
 */
#define _acount_decr(ac)                                         \
	  ({int rc;                                             \
            asm volatile(LOCKPREFIXSTR " xaddl	 %%eax,(%2)"	\
		: "=a" (rc) : "a" (-1), "r" (&(ac)->counter) : "eax"); \
	    assert(rc >= 0); \
            rc - 1; })

#include "config_inlining.h"

#ifdef CONFIG_INLINING
#define acount_init(ac, value)	_acount_init(ac, value)
#define acount_incr(ac)	_acount_incr(ac)
#define acount_decr(ac)	_acount_decr(ac)
#else /* CONFIG_INLINING */
void acount_init(struct acount *ac, int value);
int acount_incr(struct acount *ac);
int acount_decr(struct acount *ac);
#endif /* CONFIG_INLINING */

#else /* CONFIG_OPT_ACOUNT */

#include_next "acount.h"

#endif /* CONFIG_OPT_ACOUNT */

#endif /* _FLUKE_KERNEL_X86_ACOUNT_H_ */


