/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * x86-specific definitions for the context switching code.
 */
#ifndef _FLUKE_KERNEL_X86_CSW_H_
#define _FLUKE_KERNEL_X86_CSW_H_

#include "config_nonblocking.h"
#ifndef CONFIG_NONBLOCKING
#include <setjmp.h>
#endif

/*
 * Per-thread state structure maintained by the context switching layer.
 */
struct csw_state
{
	/*
	 * Page directory to load while running this thread.
	 * Kept up-to-date by the task code in s_task.c.
	 */
	unsigned		pdir_pa;

#ifndef CONFIG_NONBLOCKING
	/*
	 * Saved kernel register state for this thread.
	 */
	jmp_buf			state;
#endif
};

struct s_thread;

#include "config_opt_threads.h"
#ifdef CONFIG_OPT_THREADS
/*
 * XXX hardwired constant offset. XXX
 * Should be "&((struct gspage_seg *)0)->k.cur_thread" but gcc and gas cannot
 * seem to agree on the syntax of constants in this case.  Gcc generates
 * 	movl %ecx,%gs:$16
 * and gas doesn't like the '$' on the constant.
 */
#define csw_set_uthread_va(th) \
	asm __volatile("movl %0,%%gs:16" : : "r" ((th)->thread_va))
#else
#define csw_set_uthread_va(th)
#endif

/*
 * Set the page directory pointer for the current thread
 * and load it into the processor's PDBR register for immediate use.
 * Called only by pmap.c.
 */
void csw_set_pdir(struct s_thread *cur_thread, unsigned pdir_pa);

#include_next "csw.h"

#endif /* _FLUKE_KERNEL_X86_CSW_H_ */
