/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Common code for x86 context-switching in both
 * blocking and nonblocking kernels.
 */

#include <oskit/x86/paging.h>
#include <oskit/x86/base_tss.h>
#include <oskit/x86/base_paging.h>
#include <oskit/x86/base_stack.h>
#include "per_cpu.h"

#include "csw.h"
#include "s_thread.h"
#include "debug.h"
#include "per_cpu.h"

#include "config_kmodule.h"


extern oskit_addr_t null_pdir_pa; /* XXX layering violation */

void thread_csw_state_init(
	struct s_thread		*thread)
{
	/*
	 * New threads initially run without any task,
	 * so make them run with the kernel's null page directory.
	 */
	thread->csw_state.pdir_pa = null_pdir_pa;
}


void csw_set_pdir(struct s_thread *cur_thread, unsigned pdir_pa)
{
	assert(cur_thread == current_thread());

	cur_thread->csw_state.pdir_pa = pdir_pa;

	set_pdbr(pdir_pa);

#ifdef IOPL0
	/* set up permissions bitmask */
	if (cur_thread->task_state.task)
		bcopy(cur_thread->task_state.task->bitmask, my_bitmap,
		      65536/8);
#endif
}


void
csw_start(struct s_thread *cur_th)
{
	assert(cur_th == current_thread());

#ifdef CONFIG_KMODULE
	thread_task_state_start(cur_th); /* sets up pdir_pa properly */
	if (cur_th->csw_state.pdir_pa == base_pdir_pa) {
		/*
		 * This is a kernel-mode thread.
		 */
		extern void kmodule_thread_start(s_thread_t *);
		kmodule_thread_start(cur_th);
		otsan();
	}
#endif

	/*
	 * Switch address spaces (in fact this will always
	 * be null_pdir_pa and fault immediately to do task setup).
	 */
	csw_set_pdir(cur_th, cur_th->csw_state.pdir_pa);

	/*
	 * Change the user-visible thread identity.
	 * This is normally done by pd_thread_dispatch, but we don't
	 * call that before running the first kserver thread.
	 */
	csw_set_uthread_va(cur_th);

	/*
	 * Now we go into user mode and life becomes normal.
	 */
	csw_start_user(cur_th);
}
