/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Machine-dependent definitions for hardware interrupt handling.
 */
#ifndef _FLUKE_KERNEL_X86_INTR_H_
#define _FLUKE_KERNEL_X86_INTR_H_

/*
 * XXX should be BASE_IRQ_MASTER_BASE from <oskit/x86/pc/base_irq.h> but
 * that file cannot be included from assembly code (i.e. interrupt.S).
 */
#define IDT_IRQ_BASE	0x20

#ifndef ASSEMBLER

#include <oskit/x86/eflags.h>
#include <oskit/x86/proc_reg.h>
#include "debug.h"

struct trap_state;

#define disable_interrupts()	asm volatile("cli")
#define enable_interrupts()	asm volatile("sti")
#define save_interrupt_enable(f)	(f) = (get_eflags() & EFL_IF)
#define restore_interrupt_enable(f)	if (f) enable_interrupts()

#define assert_interrupts_enabled()	assert(get_eflags() & EFL_IF)
#define assert_interrupts_disabled()	assert(!(get_eflags() & EFL_IF))

/*
 * Attach a special low-level kernel/user interrupt handler to an IRQ line.
 * The IRQ line cannot be shared with any other handlers.
 * The handler functions are responsible for dealing with the PIC.
 * The k_func is called as a short interrupt handler -
 * thus, it can do almost nothing but call intr_preempter().
 */
int irq_alloc_ku(int irq,
	     void (*k_func)(void *data, struct trap_state *ts),
	     int (*u_func)(void *data),
	     void *data);

void irq_free_ku(int irq);

#include_next "intr.h"

#endif /* not ASSEMBLER */

#endif /* _FLUKE_KERNEL_X86_INTR_H_ */
