/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_X86_IPC_H_
#define _FLUKE_KERNEL_X86_IPC_H_

#include <fluke/ipc.h>

#define IPC_STATE_MD
struct ipc_state_md
{
	/*
	 * Save area for the minimum message received during IPC.
	 * On architectures with a reasonable number of registers,
	 * the min_msg just gets dropped straight into registers;
	 * however, on the x86, we have to save them off elsewhere
	 * and load them into registers at the very end of the receive
	 * (see IPC_FINISH_RECEIVE(), below).
	 */
	unsigned                min_msg[2];

	/*
	 * IPC parameters: only valid in certain wait states.
	 * While in WAIT_IPC_CALL, recv params are valid, for the reply.
	 * While in WAIT_IPC_ORECV, recv params are valid, for ack-send.
	 */
	fluke_ipc_params_t	params;
};

#include_next "ipc.h"

#define IPC_CONNECT_DEST(th)	EXC_SYSPARAM(th, ipc, dest)
#define IPC_STATUS(th) \
	(*((char*)&EXC_SYSPARAM(th, ipc, counts) + 2))

#define IPC_SEND_BUF_COUNT(th) \
	(*((char*)&EXC_SYSPARAM(th, ipc, counts)))
#define IPC_SEND_REF_COUNT(th) \
	(*((char*)&EXC_SYSPARAM(th, ipc, counts) + 1))

#define IPC_SEND_BUF_BUF(th)	EXC_SYSPARAM(th, ipc, send_buf_buf)
#define IPC_SEND_BUF_WORDS(th)	EXC_SYSPARAM(th, ipc, send_buf_words)
#define IPC_SEND_BUF_TAB(th)	EXC_SYSPARAM(th, ipc, send_buf_tab)
#define IPC_SEND_REF_TAB(th)	EXC_SYSPARAM(th, ipc, send_ref_tab)

#define IPC_RECV_BUF_COUNT(th) \
	(*((char*)&EXC_SYSPARAM(th, ipc, counts)))
#define IPC_RECV_REF_COUNT(th) \
	(*((char*)&EXC_SYSPARAM(th, ipc, counts) + 1))

#define IPC_RECV_BUF_BUF(th)	EXC_SYSPARAM(th, ipc, recv_buf_buf)
#define IPC_RECV_BUF_WORDS(th)	EXC_SYSPARAM(th, ipc, recv_buf_words)
#define IPC_RECV_BUF_TAB(th)	EXC_SYSPARAM(th, ipc, recv_buf_tab)
#define IPC_RECV_REF_TAB(th)	EXC_SYSPARAM(th, ipc, recv_ref_tab)

#define IPC_PARAMS(th)		EXC_SYSPARAM(th, ipc, params)

#define IPC_FINISH_RECEIVE(th, status) \
	(IPC_STATUS(th) = (status), \
	 EXC_SYSPARAM(th, ipc, recv_min_msg_0) = \
	 	(th)->ipc_state.md.min_msg[0], \
	 EXC_SYSPARAM(th, ipc, recv_min_msg_1) = \
	 	(th)->ipc_state.md.min_msg[1], \
	 EXC_SET_IP(th, fluke_nop__entry))


/* These macros indicate to MI code that certain functions are MD. */
#define MD_IPC_SEND
#define MD_IPC_CALL
#define MD_IPC_REPLY
#define MD_IPC_CLIENT_1_CONNECT_SEND
#define MD_IPC_CLIENT_1_ACK_SEND
#define MD_IPC_CLIENT_2_OVER_RECEIVE
#define MD_IPC_SERVER_2_WAIT_RECEIVE
#define MD_IPC_SERVER_2_OVER_RECEIVE
#define MD_IPC_SERVER_2_ACK_SEND
#define MD_IPC_SERVER_3_WAIT_RECEIVE

#endif /* _FLUKE_KERNEL_X86_IPC_H_ */
