/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Default implementation of blocking mutexes
 * used in blocking versions of the Fluke kernel.
 */

#include "s_thread.h"
#include "wait_queue.h"
#include "mutex.h"
#include "debug.h"

#if defined(CONFIG_KPREEMPT) && defined(CONFIG_OPT_MUTEX) && !defined(CONFIG_INLINING)

#ifdef MUTEXTIMES
#define mutex_get_tsc() \
    ({ \
	unsigned long low;
	asm volatile("rdtsc" : "=a" (low) : : "eax", "edx"); \
	low; \
    })

unsigned long mutexlockcalls, mutexlocktime;
unsigned long mutexunlockcalls, mutexunlocktime;
#endif

void mutex_lock(mutex_t *m)
{
#ifdef MUTEXTIMES
	unsigned long s, e;
	mutexlockcalls++;
	s = mutex_get_tsc();
#endif
	_mutex_lock(m);
#ifdef MUTEXTIMES
	e = mutex_get_tsc();
	mutexlocktime += (e - s);
#endif
}

void mutex_unlock(mutex_t *m)
{
#ifdef MUTEXTIMES
	unsigned long s, e;
	mutexunlockcalls++;
	s = mutex_get_tsc();
#endif
	_mutex_unlock(m);
#ifdef MUTEXTIMES
	e = mutex_get_tsc();
	mutexunlocktime += (e - s);
#endif
}
#endif /* CONFIG_KPREEMPT && CONFIG_OPT_MUTEX && MD_MUTEX_PROFILE */


