/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_X86_U_REF_H_
#define _FLUKE_KERNEL_X86_U_REF_H_

#include_next "u_ref.h"

/*
 * Given a s_ob_t pointer, construct a "handle value" that both encodes
 * the flob type and is unique among all flobs on the system.
 *
 * We use the kernel-mode pointer itself as the unique basis for the handle.
 * Then we squeeze the type bits in by taking advantage of the fact that
 * the flob allocator aligns all s_ob_t's on power-of-two boundaries and
 * the smallest boundary is at 64 bytes, leaving the low four bits clear.
 */
#define REF_HANDLE(sobp) ({						      \
	oskit_addr_t _handle;						      \
	assert(((sobp)->type &~ 0xf) == 0);				      \
	_handle = (oskit_addr_t)(sobp);					      \
	assert((_handle & 0xf) == 0);					      \
	_handle |= (sobp)->type;					      \
})

#endif /* _FLUKE_KERNEL_X86_U_REF_H_ */
