/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_FLUKE_OPT_SYNCH_MUTEX_H
#define _FLUKE_KERNEL_FLUKE_OPT_SYNCH_MUTEX_H

/*
 * We define here the format of the data (a single state word)
 * stored in the user-space part of mutex flobs in the optimized kernel.
 * The mutex_lock and mutex_unlock Fluke API entrypoints atomically
 * frob this word and only enter the kernel when they need to block
 * or to unblock other threads.
 */

typedef struct u_mutex
{
	int	state;
#define U_MUTEX_AVAILABLE	0 /* locked=0 */
#define U_MUTEX_HELD		1 /* locked=1, noone in mutex_lock */
#define U_MUTEX_CONTESTED	-1 /* locked=1, someone is in mutex_lock */
} u_mutex_t;

#define MUTEX_INITLOCK(m)		((m)->u_mutex->state = 0)
#define MUTEX_ISLOCKED(m)		((m)->u_mutex->state != 0)
#define MUTEX_SETLOCKED(m, flag)					      \
({									      \
	int ostate;							      \
	asm("xchgl %1,%0 # %2"	/* xchg by itself is always atomic */	      \
	    : "=m" ((m)->u_mutex->state), "=r" (ostate)			      \
	    : "1" ((flag) ? U_MUTEX_CONTESTED : U_MUTEX_AVAILABLE));	      \
})

#define MD_SYS_MUTEX_TRYLOCK	/* don't define standard kernel trap at all */
#define MD_S_MUTEX_LOCKOP
#define MD_S_MUTEX_UNLOCKOP

#include "config_opt_synch.h"
#include_next "s_mutex.h"

#endif /* _FLUKE_KERNEL_FLUKE_OPT_SYNCH_MUTEX_H */
