/*
 * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#define UENTRY_KPAGE	EXT(u_thread_page)
#define UENTRY_UPAGE	0x04000000
#include "u_entry.h"

#include "config_profiling.h"


#ifdef ASSEMBLER

#include <oskit/x86/asm.h>

#include "x86.h"
#include "idt.h"
#include "s_thread_asm.h"

	P2ALIGN(12)

	.globl	EXT(u_thread_page)
LEXT(u_thread_page)

#endif /* ASSEMBLER */

/* 0x000 */
#ifdef CONFIG_PROFILING
FLUKE_ENTRY(sys_doprof)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_debug_puts)
FLUKE_ENTRY(sys_debug_dump_flob)
FLUKE_ENTRY(sys_debug_label_flob)

/* Optimized implementation of fluke_thread_get_saved_state() (64 bytes) */
BEGIN_UENTRY(thread_get_saved_state)
	MOV_UTH_REG(esi, EAX)
	MOV_UTH_REG(ecx, ECX)
	MOV_UTH_REG(edx, EDX)
	MOV_UTH_REG(edi, EIP)
	MOV_UTH_REG(ebx, EFLAGS)
	RET
END_UENTRY(thread_get_saved_state, 0x40)

FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* Optimized implementation of fluke_thread_set_saved_state() (64 bytes) */
BEGIN_UENTRY(thread_set_saved_state)
	MOV_REG_UTH(esi, EAX)
	MOV_REG_UTH(ecx, ECX)
	MOV_REG_UTH(edx, EDX)
	MOV_REG_UTH(edi, EIP)
	MOV_REG_UTH(ebx, EFLAGS)
	RET
END_UENTRY(thread_set_saved_state, 0x40)

/* 0x100 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

#if 1 /* XXX needs to check for pending exceptions, not clobber stack */
FLUKE_ENTRY(sys_thread_return_from_exception)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
#else
/* Optimized fluke_thread_return_from_exception() (64 bytes) */
BEGIN_UENTRY(thread_return_from_exception)
	orl	$FLUKE_THREAD_EXCEPTION_ENABLE,UTH_LFLAGS(%eax)
	pushl	UTH_SAVED_EIP(%eax)
	pushl	UTH_SAVED_EFLAGS(%eax)
	MOV_UTH_REG(ecx, ECX)
	MOV_UTH_REG(edx, EDX)
	MOV_UTH_REG(eax, EAX)
	popf
	RET
END_UENTRY(thread_return_from_exception, 0x40)
#endif

FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* Optimized implementation of fluke_thread_set_handlers() (64 bytes) */
BEGIN_UENTRY(thread_set_handlers)
	MOV_REG_HDLR(ecx, TRAP)
	MOV_REG_HDLR(edx, INTERRUPT)
	MOV_REG_HDLR(esi, CLIENT_ALERT)
	MOV_REG_HDLR(edi, SERVER_ALERT)
	RET
END_UENTRY(thread_set_handlers, 0x40)

FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x200 */
/* Optimized implementation of fluke_thread_get_handlers() (64 bytes) */
BEGIN_UENTRY(thread_get_handlers)
	MOV_HDLR_REG(ecx, TRAP)
	MOV_HDLR_REG(edx, INTERRUPT)
	MOV_HDLR_REG(esi, CLIENT_ALERT)
	MOV_HDLR_REG(edi, SERVER_ALERT)
	RET
END_UENTRY(thread_get_handlers, 0x40)

FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_get_client)
#if FLASK
FLUKE_ENTRY(sys_thread_get_client_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_thread_set_client)
#if FLASK
FLUKE_ENTRY(sys_thread_set_client_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_thread_get_server)
#if FLASK
FLUKE_ENTRY(sys_thread_get_server_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_thread_set_server)
#if FLASK
FLUKE_ENTRY(sys_thread_set_server_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif

/* 0x300 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_get_state)
FLUKE_ENTRY(sys_invalid)

/* 0x400 */

#if 1 /* XXX needs to check for pending exceptions */
FLUKE_ENTRY(sys_thread_enable_exceptions)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
#else
/* Optimized implementation of fluke_thread_enable_exceptions() (64 bytes) */
BEGIN_UENTRY(thread_enable_exceptions)
	THVA_REG(eax)
	SET_LFLAGS(%eax, FLUKE_THREAD_EXCEPTION_ENABLE)
	RET
END_UENTRY(thread_enable_exceptions, 0x40)
#endif

/* Optimized implementation of fluke_thread_disable_exceptions() (64 bytes) */
BEGIN_UENTRY(thread_disable_exceptions)
	THVA_REG(eax)
	CLEAR_LFLAGS(%eax, FLUKE_THREAD_EXCEPTION_ENABLE)
	RET
END_UENTRY(thread_disable_exceptions, 0x40)

FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_interrupt)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* Optimized implementation of fluke_thread_self() (32 bytes) */
BEGIN_UENTRY(thread_self)
	THVA_REG(eax)
	RET
END_UENTRY(thread_self, 0x20)

/* 0x500 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_move)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_reference)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x600 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_move)
FLUKE_ENTRY(sys_invalid)

/* 0x700 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_reference)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x800 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_move)

/* 0x900 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_protect)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xa00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_get_state)
FLUKE_ENTRY(sys_invalid)

/* 0xb00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_move)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_reference)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_protect)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_search)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xc00 */
FLUKE_ENTRY(sys_thread_yield)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xd00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xe00 */
FLUKE_ENTRY(sys_instrument_thread) /* empty (valid) syscall ifndef CONFIG_INSTRUMENT */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xf00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_special) /* 0xfe0: Kernel private, not in the Fluke API */

#ifdef ASSEMBLER
/* 0xff0: sys_nop (special) */
	ret
	.byte	0
	.short	0,0,0,0,0
	.long	EXT(sys_invalid)
#endif
