/*
 * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef KERNEL_X86_PER_CPU_H
#define KERNEL_X86_PER_CPU_H

#include "config_smp.h"
#include "config_user_sched.h"

/* This contains the per-cpu declarations. */
#include <oskit/x86/base_gdt.h>
#include <oskit/x86/base_idt.h>
#include <oskit/x86/base_tss.h>

#include "preempt.h"

/*
 * Here I declare the per-cpu structure.
 */
typedef struct fluke_cpu_info {
	struct x86_desc gdt[GDTSZ];
	struct x86_tss tss;
#ifdef IOPL0
	char bitmap[65536/8 + 1]; /* IO Bitmap */
#endif

#ifdef CONFIG_SMP
	int number;		/* CPU number for smp_* calls */

	struct fluke_cpu_info *next; /* circular list of all CPUs */

	volatile int inval_tlb;	/* pending TLB flush */
	volatile int cancel_pending; /* pending user thread cancel */
	volatile int yield_pending; /* flag requests yield to provider */
#endif

#if !defined(CONFIG_NOPREEMPT) && !defined(CONFIG_USER_SCHED)
	volatile int reschedule; /* reschedule request flag */
#endif

	/*
	 * These members are used only by this CPU itself.
	 */
	struct preempter *preempters; /* registered preempters to be called */
#if !defined(CONFIG_NOPREEMPT) && !defined(CONFIG_USER_SCHED)
	struct preempter disp_preempter; /* used with disp_reschedule */
#endif
#ifdef CONFIG_KPREEMPT
	int preemption_enabled;
#endif

} fluke_cpu_info_t;

#include "spin_lock.h"

/*
 * This lock protects the members used for interprocessor communication.
 */
extern spin_lock_t ipi_lock; /* apic.c */


/*
 * This is the per-CPU structure for the bootstrap (i.e. first) processor.
 * The symbols base_gdt, base_idt, and base_tss actually point to
 * the members of the boot_cpu_info structure.  The cpu_info structures
 * for other processors are dynamically allocated.
 */
extern fluke_cpu_info_t boot_cpu_info;

/*
 * The macro `my_cpu' always refers to the current CPU
 * that the current thread is running on.  These macros
 * are more shorthand for its members.
 */
#define my_gdt (my_cpu.gdt)
#define my_idt (my_cpu.idt)
#define my_tss (my_cpu.tss)
#ifdef IOPL0
#define my_bitmap (my_cpu.bitmap)
#endif

#ifndef CONFIG_SMP

#define	my_cpu	boot_cpu_info

#define MAX_CPUS 1

#define smp_reschedule()

#else

#if 1 /*ndef CONFIG_OPT_THREADS */

#define	my_cpu	(*current_thread()->disp_state.cpu)

#include "s_thread.h"		/* for current_thread() */

#else /* CONFIG_OPT_THREADS */
/*
 * A word in the kernel-only portion of the GSPAGE points to the cpu_info
 * structure.  Since the %gs segment register on each CPU always refers to
 * the per-processor GSPAGE, this same segmented address viewed from each
 * CPU points to that CPU's own cpu_info pointer.
 */
#define my_cpu	(*current_cpu)

/*
 * We can just declare the constant segmented address as a global variable
 * with a funny asm name; since we declare the variable const, the compiler
 * will optimize references happily.
 */
#define PASTE(a,b) PASTE_1(a,b)
#define PASTE_1(a,b) a ## #b
extern fluke_cpu_info_t *const current_cpu __asm__(PASTE("%gs:",GS_CPU_INFO));
#endif /* !CONFIG_OPT_THREADS */

#include <oskit/smp.h>

#define MAX_CPUS 15	/* Integrated APIC restricts us to at most 15 */

#endif CONFIG_SMP

struct fluke_cpu_info *per_cpu_init(int cpu_number);
void per_cpu_load(struct fluke_cpu_info *);

#endif KERNEL_X86_PER_CPU_H
