/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This file provides the definition for the startup_thread,
 * which is the pseudo-thread the kernel uses while getting started.
 * The initial (base) stack is contained in this thread in the normal place,
 * and the startup_thread is properly aligned as threads should be;
 * this way current_thread() will work properly during startup.
 */

#include <oskit/x86/asm.h>

#include "config_nonblocking.h"

#include "x86.h"

	.bss

	P2ALIGN(S_THREAD_ALIGN)
	.globl	EXT(startup_thread)
LEXT(startup_thread)
	.ds.b	S_THREAD_SIZE

/*
 * The OS toolkit normally provides a default startup stack
 * of size BASE_STACK_SIZE (see oskit/x86/base_stack.h)
 * with these symbols demarking its beginning and end;
 * here we override it with our own stack to suit our requirements.
 */

	.globl	EXT(base_stack_start)
	.globl	EXT(base_stack_end)

#ifdef CONFIG_NONBLOCKING

	/*
	 * In the nonblocking kernel,
	 * the base kernel stack needs to be naturally aligned to its size,
	 * so that the current_thread variable at offset zero in the stack
	 * can be found quickly (see x86/s_thread.h).
	 */
	P2ALIGN(PROCESSOR_STACK_SHIFT)

LEXT(base_stack_start)
	.space	PROCESSOR_STACK_SIZE
LEXT(base_stack_end)

#else /* not CONFIG_NONBLOCKING */

	/*
	 * In the blocking kernel,
	 * the kernel stack for a thread is located in the thread itself,
	 * so the base stack needs to be located in the startup thread.
	 */
	.equ	EXT(base_stack_start),EXT(startup_thread)+TH_STACK
	.equ	EXT(base_stack_end),EXT(startup_thread)+TH_EXC_STATE

#endif /* not CONFIG_NONBLOCKING */

